/*
* Copyright 1993-2009 NVIDIA Corporation.  All rights reserved.
*
* NOTICE TO USER:   
*
* This source code is subject to NVIDIA ownership rights under U.S. and 
* international Copyright laws.  
*
* NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE 
* CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR 
* IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH 
* REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF 
* MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.   
* IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL, 
* OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS 
* OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
* OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE 
* OR PERFORMANCE OF THIS SOURCE CODE.  
*
* U.S. Government End Users.  This source code is a "commercial item" as 
* that term is defined at 48 C.F.R. 2.101 (OCT 1995), consisting  of 
* "commercial computer software" and "commercial computer software 
* documentation" as such terms are used in 48 C.F.R. 12.212 (SEPT 1995) 
* and is provided to the U.S. Government only as a commercial end item.  
* Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through 
* 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the 
* source code with only those rights set forth herein.
*/
#ifndef NV_NPPI_H
#define NV_NPPI_H
 
/**
 * \file nppi.h
 * NPP Image Processing Functionality.
 */
 
 #include "nppdefs.h"


#ifdef __cplusplus
extern "C" {
#endif

/** \defgroup nppi NPP Image Processing
 * @{
 */


/** @name 2D Memory Allocation
 *  Allocator methods for 2D arrays of data. The allocators have width and height parameters
 * to specify the size of the 2D array (image) to be allocated. They return a pointer to the
 * newly created memory and return the numbers of bytes between successive lines. 
 */
//@{

/**
 * 8bit unsigned, single-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp8u  * nppiMalloc_8u_C1(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 8bit unsigned, two-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp8u  * nppiMalloc_8u_C2(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 8bit unsigned, three-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp8u  * nppiMalloc_8u_C3(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 8bit unsigned, four-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp8u  * nppiMalloc_8u_C4(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 16bit unsigned, single-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp16u * nppiMalloc_16u_C1(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 16bit unsigned, triple-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp16u * nppiMalloc_16u_C3(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 16bit unsigned, four-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp16u * nppiMalloc_16u_C4(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 16bit signed, single-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp16s * nppiMalloc_16s_C1(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 16bit signed, four-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp16s * nppiMalloc_16s_C4(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 32bit signed, single-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp32s * nppiMalloc_32s_C1(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 32bit signed, triple-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp32s * nppiMalloc_32s_C3(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 32bit signed, quad-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp32s * nppiMalloc_32s_C4(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 32bit floating-point, single-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp32f * nppiMalloc_32f_C1(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 32bit floating-point, triple-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp32f * nppiMalloc_32f_C3(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * 32bit floating-point, quad-channel 2D (image) memory allocator.
 * \param nWidthPixels The width of the 2D array (image) to be allocated.
 * \param nHeightPixels The height of the 2D array (image) to be allocated.
 * \param pStepBytes The number of bytes between successive rows of pixels is returned via this pointer to int.
 * \return A pointer to the new 2D array (image). 0 (null-pointer) indicates that an error occurred
 *      during allocation.
 */
Npp32f * nppiMalloc_32f_C4(int nWidthPixels, int nHeightPixels, int * pStepBytes);

/**
 * Free method for any 2D allocated memory.
 * This method should be used to free memory allocated with any of the nppiMalloc_<modifier> methods.
 * \param pData A pointer to memory allocated using nppiMalloc_<modifier>.
 */
void nppiFree(void * pData);
//@}


/** @name 2D Memory Set
 * Set methods for 2D images of various types. The set methods operate on image data given
 * as a pointer to the underlying pixels-channel data-type (e.g. four-channel 8-bit images would
 * be passed as pointers to Npp8u type) and a step-width, i.e. the number of bytes between sucessive
 * line starts. The size of the area to be set is specified via a NppiSize struct. All methods also
 * take the pixel value to be set. In case of single channel images this is a single value, in case
 * of multi-channel, a vector of values gets passed. 
 */
//@{

/** 
 * 8bit unsigned integer, single channel (image) data set method.
 * \param nValue The pixel value to be set.
 * \param pDst Pointer to the start pixel in the region-of-interest.
 * \param nDstStep Number of bytes between line starts of successive lines.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nDstStep is null or negative or if the ROI-width exceeds the step-size.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiSet_8u_C1R( Npp8u nValue, Npp8u * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 8bit unsigned integer, four channel (image) data set method.
 * \param aValues Four-channel array containing the pixel-value to be set.
 * \param pDst Pointer to the start pixel in the region-of-interest.
 * \param nDstStep Number of bytes between line starts of successive lines.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nDstStep is null or negative or if the ROI-width exceeds the step-size.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiSet_8u_C4R( const Npp8u aValues[4], Npp8u * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 8bit unsigned integer, four channel (image) data set method, not affecting Alpha.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * \param aValues Three-channel array containing the pixel-value to be set.
 * \param pDst Pointer to the start pixel in the region-of-interest.
 * \param nDstStep Number of bytes between line starts of successive lines.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nDstStep is null or negative or if the ROI-width exceeds the step-size.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiSet_8u_AC4R(const Npp8u aValues[3], Npp8u * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 32bit foat, single channel (image) data set method.
 * \param nValue The pixel-value to be set.
 * \param pDst Pointer to the start pixel in the region-of-interest.
 * \param nDstStep Number of bytes between line starts of successive lines.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nDstStep is null or negative or if the ROI-width exceeds the step-size.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiSet_32f_C1R(Npp32f nValue, Npp32f * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 32bit signed integer, single channel (image) data set method.
 * \param nValue The pixel-value to be set.
 * \param pDst Pointer to the start pixel in the region-of-interest.
 * \param nDstStep Number of bytes between line starts of successive lines.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nDstStep is null or negative or if the ROI-width exceeds the step-size.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiSet_32s_C1R(Npp32s nValue, Npp32s * pDst, int nDstStep, NppiSize oSizeROI);
//@}


/** @name Bit-Depth Conversion
 * Convert bit-depth up and down.
 *
 * The integer conversion methods do not involve any scaling. Conversions that reduce bit-deptch saturate
 * values exceeding the reduced range to the range's maximum/minimum value.
 * When converting from floating-point values to integer values, a rounding mode can be specifed. After rounding
 * to integer values the values get saturated to the destination data type's range.
 */
//@{


/** 
 * 8-bit unsigned integer to 16-bit unsigned integer, single channel conversion.
 *
 * \param pSrc Pointer to the source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrcStep Number of bytes between line starts of successive lines in source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pSrc or pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiConvert_8u16u_C1R(const Npp8u  * pSrc, int nSrcStep, Npp16u * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 16-bit unsigned integer to 8-bit unsigned integer, single channel conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 */
NppStatus nppiConvert_16u8u_C1R(const Npp16u * pSrc, int nSrcStep, Npp8u  * pDst, int nDstStep, NppiSize oSizeROI);


/** 
 * 8-bit unsigned integer to 16-bit unsigned integer, four channel conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 */
NppStatus nppiConvert_8u16u_C4R(const Npp8u  * pSrc, int nSrcStep, Npp16u * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 16-bit unsigned integer to 8-bit unsigned integer, four channel conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 */
NppStatus nppiConvert_16u8u_C4R(const Npp16u * pSrc, int nSrcStep, Npp8u  * pDst, int nDstStep, NppiSize oSizeROI);


/** 
 * 8-bit unsigned integer to 16-bit unsigned integer, four channel conversion not affecting alpha.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 */
NppStatus nppiConvert_8u16u_AC4R(const Npp8u  * pSrc, int nSrcStep, Npp16u * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 16-bit unsigned integer to 8-bit unsigned integer, four channel conversion not affecting alpha.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 */
NppStatus nppiConvert_16u8u_AC4R(const Npp16u * pSrc, int nSrcStep, Npp8u  * pDst, int nDstStep, NppiSize oSizeROI);
          

/** 
 * 8-bit unsigned integer to 16-bit singed integer, single channel conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 */
NppStatus nppiConvert_8u16s_C1R(const Npp8u  * pSrc, int nSrcStep, Npp16s * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 16-bit signed integer to 8-bit unsigned integer, four single conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 */
NppStatus nppiConvert_16s8u_C1R(const Npp16s * pSrc, int nSrcStep, Npp8u  * pDst, int nDstStep, NppiSize oSizeROI);
          

/** 
 * 8-bit unsigned integer to 16-bit signed integer, four channel conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 */
NppStatus nppiConvert_8u16s_C4R(const Npp8u  * pSrc, int nSrcStep, Npp16s * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 16-bit signed integer to 8-bit unsigned integer, four channel conversion not affecting alpha.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 */
NppStatus nppiConvert_16s8u_C4R(const Npp16s * pSrc, int nSrcStep, Npp8u  * pDst, int nDstStep, NppiSize oSizeROI);


/** 
 * 8-bit unsigned integer to 16-bit signed integer, four channel conversion not affecting alpha.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 */
NppStatus nppiConvert_8u16s_AC4R(const Npp8u  * pSrc, int nSrcStep, Npp16s * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 16-bit signed integer to 8-bit unsigned integer, four channel conversion not affecting alpha.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 */
NppStatus nppiConvert_16s8u_AC4R(const Npp16s * pSrc, int nSrcStep, Npp8u  * pDst, int nDstStep, NppiSize oSizeROI);
          
          
/** 
 * 16-bit singed integer to 32-bit float, single channel conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 */
NppStatus nppiConvert_16s32f_C1R(const Npp16s * pSrc, int nSrcStep, Npp32f * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 32-bit float to 16-bit signed integer, single channel conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * \param pSrc Pointer to source image.
 * \param nSrcStep Source-image line step.
 * \param pDst Pointer to destination image.
 * \param nDstStep Destination-image line step.
 * \param oSizeROI Size of the region-of-interest.
 * \param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 */
NppStatus nppiConvert_32f16s_C1R(const Npp32f * pSrc, int nSrcStep, Npp16s * pDst, int nDstStep, NppiSize oSizeROI, NppRoundMode eRoundMode);


/** 
 * 16-bit unsinged integer to 32-bit float, single channel conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * \param pSrc Pointer to source image.
 * \param nSrcStep Source-image line step.
 * \param pDst Pointer to destination image.
 * \param nDstStep Destination-image line step.
 * \param oSizeROI Size of the region-of-interest.
 */
NppStatus nppiConvert_16u32f_C1R(const Npp16u * pSrc, int nSrcStep, Npp32f * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 32-bit float to 16-bit unsigned integer, single channel conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * \param pSrc Pointer to source image.
 * \param nSrcStep Source-image line step.
 * \param pDst Pointer to destination image.
 * \param nDstStep Destination-image line step.
 * \param oSizeROI Size of the region-of-interest.
 * \param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 */
NppStatus nppiConvert_32f16u_C1R(const Npp32f * pSrc, int nSrcStep, Npp16u * pDst, int nDstStep, NppiSize oSizeROI, NppRoundMode eRoundMode);


/** 
 * 16-bit unsinged integer to 32-bit signed, single channel conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * \param pSrc Pointer to source image.
 * \param nSrcStep Source-image line step.
 * \param pDst Pointer to destination image.
 * \param nDstStep Destination-image line step.
 * \param oSizeROI Size of the region-of-interest.
 */
NppStatus nppiConvert_16u32s_C1R(const Npp16u * pSrc, int nSrcStep, Npp32s * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 16-bit signed 32-bit signed integer, single channel conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * \param pSrc Pointer to source image.
 * \param nSrcStep Source-image line step.
 * \param pDst Pointer to destination image.
 * \param nDstStep Destination-image line step.
 * \param oSizeROI Size of the region-of-interest.
 */
NppStatus nppiConvert_16s32s_C1R(const Npp16s * pSrc, int nSrcStep, Npp32s * pDst, int nDstStep, NppiSize oSizeROI);

//@}


/** @name 2D Memory Copy
 * Copy methods for 2D images of various types. The copy methods operate on image data given
 * as a pointer to the underlying pixels-channel data-type (e.g. four-channel 8-bit images would
 * be passed as pointers to Npp8u type) and a step-width, i.e. the number of bytes between sucessive
 * line starts. The size of the area to be copied is specified via a NppiSize struct. 
 */
//@{

/** 
 * 8bit unsigned integer, single channel (image) copy method.
 * \param pSrc Pointer to the source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrcStep Number of bytes between line starts of successive lines in source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pSrc or pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiCopy_8u_C1R(const Npp8u * pSrc, int nSrcStep, Npp8u * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 8bit unsigned integer, four channel (image) copy method.
 * \param pSrc Pointer to the source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrcStep Number of bytes between line starts of successive lines in source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pSrc or pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiCopy_8u_C4R(const Npp8u * pSrc, int nSrcStep, Npp8u * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 8bit unsigned integer, four channel (image) copy method, not affecting Alpha channel.
 * This is for copying the color portion of RGBA images, leaving the destination image's alpha information
 * intact.
 * \param pSrc Pointer to the source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrcStep Number of bytes between line starts of successive lines in source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pSrc or pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiCopy_8u_AC4R(const Npp8u * pSrc, int nSrcStep, Npp8u * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 32bit float, single channel (image) copy method.
 * \param pSrc Pointer to the source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrcStep Number of bytes between line starts of successive lines in source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pSrc or pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiCopy_32f_C1R(const Npp32f * pSrc, int nSrcStep, Npp32f * pDst, int nDstStep, NppiSize oSizeROI);

/** 
 * 32bit signed integer, single channel (image) copy method.
 * \param pSrc Pointer to the source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrcStep Number of bytes between line starts of successive lines in source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If pSrc or pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiCopy_32s_C1R(const Npp32s * pSrc, int nSrcStep, Npp32s * pDst, int nDstStep, NppiSize oSizeROI);
//@}


/** @name Copy Const Border
 * Methodes for copying images and padding borders with a constant, user-specifiable color.
 */
//@{

/** 
 * 8bit unsigned integer, single channel (image) copy widht constant border color.
 * \param pSrc Pointer to the source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrcStep Number of bytes between line starts of successive lines in source image..
 * \param oSrcSizeROI Size of the source region of pixels.
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and constant border color (outer part).
 * \param nTopBorderHeight Height (in pixels) of the top border. The height of the border at the bottom of the
 *      destination ROI is implicitly defined by the size of the source ROI: 
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * \param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destinaiton ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * \param nValue The pixel value to be set for border pixels.
 * \return NPP_NULL_POINTER_ERROR If pSrc or pDst is the null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiCopyConstBorder_8u_C1R(const Npp8u * pSrc,  int nSrcStep, NppiSize oSrcSizeROI,
                                             Npp8u * pDst,  int nDstStep, NppiSize oDstSizeROI,
                                       int nTopBorderHeight, int nLeftBorderWidth,
                                       Npp8u nValue);

/**
 * 8-bit unsigned integer, four channel (image) copy with constant border color.
 * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
 * \param pSrc Pointer to source image.
 * \param nSrcStep Source-image line step.
 * \param oSrcSizeROI Size of the source region-of-interest.
 * \param pDst Pointer to destination image.
 * \param nDstStep Destination-image line step.
 * \param oDstSizeROI Size of the destination region-of-interest.
 * \param nTopBorderHeight Hight of top border.
 * \param nLeftBorderWidth Width of left border.
 * \param aValue Vector of the RGBA values of the border pixels to be set.
 */
NppStatus nppiCopyConstBorder_8u_C4R(const Npp8u * pSrc, int nSrcStep, NppiSize oSrcSizeROI,
                                             Npp8u * pDst, int nDstStep, NppiSize oDstSizeROI,
                                       int nTopBorderHeight, int nLeftBorderWidth,
                                       const Npp8u aValue[4]);
                                       
/**
 * 8-bit unsigned integer, four channel (image) copy with constant border color.
 * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
 * \param pSrc Pointer to source image.
 * \param nSrcStep Source-image line step.
 * \param oSrcSizeROI Size of the source region-of-interest.
 * \param pDst Pointer to destination image.
 * \param nDstStep Destination-image line step.
 * \param oDstSizeROI Size of the destination region-of-interest.
 * \param nTopBorderHeight Hight of top border.
 * \param nLeftBorderWidth Width of left border.
 * \param aValue Vector of the RGB values of the border pixels. Because this method doesn't
 *      affect the destination image's alpha channel, only three components of the border color
 *      are needed.
 */
NppStatus nppiCopyConstBorder_8u_AC4R(const Npp8u * pSrc,  int nSrcStep, NppiSize oSrcSizeROI,
                                              Npp8u * pDst,  int nDstStep, NppiSize oDstSizeROI,
                                        int nTopBorderHeight, int nLeftBorderWidth,
                                        const Npp8u aValue[3]);

/* 32-bit signed integer, single channel (image) copy with constant border color.
 * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
  * \param pSrc Pointer to source image.
 * \param nSrcStep Source-image line step.
 * \param oSrcSizeROI Size of the source region-of-interest.
 * \param pDst Pointer to destination image.
 * \param nDstStep Destination-image line step.
 * \param oDstSizeROI Size of the destination region-of-interest.
 * \param nTopBorderHeight Hight of top border.
 * \param nLeftBorderWidth Width of left border.
 * \param nValue Border luminance value.
*/
NppStatus nppiCopyConstBorder_32s_C1R(const Npp32s * pSrc, int nSrcStep, NppiSize oSrcSizeROI,
                                              Npp32s * pDst, int nDstStep, NppiSize oDstSizeROI,
                                        int nTopBorderHeight, int nLeftBorderWidth,
                                        Npp32s nValue);

//@}

/** @name Image Transpose
 * Methods for transposing 2D images of various types. Like matrix transpose, image transpose is a mirror on the image's
 * diagonal (uppler-left to lower-right corner).
 */
//@{

/**
 * Transpose 8bit single channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oROI Width and height of the regions of interest.
 *
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR For successful completetion.
 */
NppStatus nppiTranspose_8u_C1R(const Npp8u * pSrc, int nSrcStep, Npp8u * pDst, int nDstStep, NppiSize oROI);
//@}


/** @name Image Color Channel Swap
 * Methods for exchaning the color channels of an image. The methods support arbitrary permutations of the original
 * channels, including replication.
 */
//@{

/**
 *  8bit unsigned integer, four channel swap channels, in-place.
 *
 * \param pSrcDst Pointer to the image ROI.
 * \param nSrcDstStep Distance in bytes between starts of consecutive lines of the image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param aDstOrder Integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [3,2,1,0] converts this to ABGR
 *      channel order.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_SUCCESS For successful completetion.
 */
NppStatus nppiSwapChannels_8u_C4IR(Npp8u * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, const int aDstOrder[4]);
//@}


/** @name Image Addition Methods
 * Methods for adding two images.
 */
//@{

/** 
 * 8bit unsigned integer, single channel (image) add.
 * Add the pixel values of corresponging pixels in the ROI and write them to the output image.
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiAdd_8u_C1RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                              const Npp8u * pSrc2, int nSrc2Step, 
                                    Npp8u * pDst,  int nDstStep, 
                              NppiSize oSizeROI, int nScaleFactor);

/** 
 * 8bit unsigned integer, four channel (image) add.
 * Add the pixel values of corresponging pixels in the ROI and write them to the output image.
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiAdd_8u_C4RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                              const Npp8u * pSrc2, int nSrc2Step, 
                                    Npp8u * pDst,  int nDstStep, 
                              NppiSize oSizeROI, int nScaleFactor);

/** 
 * 8bit unsigned integer, four channel (image) add, not affecting alpha channel.
 * Add the pixel values of corresponging pixels in the ROI and write them to the output image.
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiAdd_8u_AC4RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                               const Npp8u * pSrc2, int nSrc2Step, 
                                     Npp8u * pDst,  int nDstStep, 
                               NppiSize oSizeROI, int nScaleFactor);

/** 
 * 32bit float, single channel (image) add.
 * Add the pixel values of corresponging pixels in the ROI and write them to the output image.
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiAdd_32f_C1R(const Npp32f * pSrc1, int nSrc1Step, 
                            const Npp32f * pSrc2, int nSrc2Step, 
                                  Npp32f * pDst,  int nDstStep, 
                            NppiSize oSizeROI);                        
//@}

/** @name Image Subtraction Methods
 * Methods for subracting one image from another.
 */
//@{

/** 
 * 8bit unsigned integer, single channel (image) sub.
 * Subtrace the pixel values of corresponging pixels in the ROI and write them to the output image.
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiSub_8u_C1RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                              const Npp8u * pSrc2, int nSrc2Step, 
                                    Npp8u * pDst,  int nDstStep, 
                              NppiSize oSizeROI, int nScaleFactor);

/** 
 * 8bit unsigned integer, four channel (image) subtraction.
 * Subtract pSrc1's pixels from corresponging pixels in pSrc2. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiSub_8u_C4RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                              const Npp8u * pSrc2, int nSrc2Step, 
                                    Npp8u * pDst,  int nDstStep, 
                              NppiSize oSizeROI, int nScaleFactor);

/** 
 * 8bit unsigned integer, four channel (image) subtraction, not affecting alpha channel.
 * Subtract pSrc1's pixels from corresponging pixels in pSrc2. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiSub_8u_AC4RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                               const Npp8u * pSrc2, int nSrc2Step, 
                                     Npp8u * pDst, int nDstStep, 
                               NppiSize oSizeROI, int nScaleFactor);

/** 
 * 32bit float, single channel (image) subtraction.
 * Subtract pSrc1's pixels from corresponging pixels in pSrc2. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiSub_32f_C1R(const Npp32f * pSrc1, int nSrc1Step, 
                            const Npp32f * pSrc2, int nSrc2Step, 
                                  Npp32f * pDst,  int nDstStep, 
                            NppiSize oSizeROI);
//@}

/** @name Image Multiplication Methods
 * Methods for multiplying two images.
 */
//@{

/** 
 * 8bit unsigned integer, single channel (image) mul.
 * Multiply the pixel values of corresponging pixels in the ROI and write them to the output image.
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiMul_8u_C1RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                              const Npp8u * pSrc2, int nSrc2Step, 
                                    Npp8u * pDst,  int nDstStep, 
                              NppiSize oSizeROI, int nScaleFactor);

/** 
 * 8bit unsigned integer, four channel (image) multiplication.
 * Multiply corresponging pixels in ROI. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiMul_8u_C4RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                              const Npp8u * pSrc2, int nSrc2Step, 
                                    Npp8u * pDst,  int nDstStep, 
                              NppiSize oSizeROI, int nScaleFactor);
                              
/** 
 * 8bit unsigned integer, four channel (image) multiplication, not affecting alpha channel.
 * Multiply corresponging pixels in ROI. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiMul_8u_AC4RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                               const Npp8u * pSrc2, int nSrc2Step, 
                                     Npp8u * pDst,  int nDstStep, 
                               NppiSize oSizeROI, int nScaleFactor);

/** 
 * 32bit float, singlechannel (image) multiplication.
 * Multiply corresponging pixels in ROI. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiMul_32f_C1R(const Npp32f * pSrc1, int nSrc1Step, 
                            const Npp32f * pSrc2, int nSrc2Step, 
                                  Npp32f * pDst,  int nDstStep, 
                            NppiSize oSizeROI); 
//@}

/** @name Image Division Methods
 * Methods for dividing one image by another.
 */
//@{

/** 
 * 8bit unsigned integer, single channel (image) div.
 * Dived pixels in pSrc2 by pSrc1's pixels.
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiDiv_8u_C1RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                              const Npp8u * pSrc2, int nSrc2Step, 
                                    Npp8u * pDst,  int nDstStep, 
                              NppiSize oSizeROI, int nScaleFactor);

/** 
 * 8bit unsigned integer, four channel (image) division.
 * Divide pixels in pSrc2 by pSrc1's pixels. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiDiv_8u_C4RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                              const Npp8u * pSrc2, int nSrc2Step, 
                                    Npp8u * pDst,  int nDstStep, 
                              NppiSize oSizeROI, int nScaleFactor);

/** 
 * 8bit unsigned integer, four channel (image) division, not affecting alpha channel.
 * Divide pixels in pSrc2 by pSrc1's pixels. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nScaleFactor Result pixel values are scaled by 2^(-nScaleFactor) and then clamped to [0,255] range.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiDiv_8u_AC4RSfs(const Npp8u * pSrc1, int nSrc1Step, 
                               const Npp8u * pSrc2, int nSrc2Step, 
                                     Npp8u * pDst,  int nDstStep, 
                               NppiSize oSizeROI, int nScaleFactor);


NppStatus nppiAbsDiff_8u_C4R(const Npp8u * pSrc1, int nSrc1Step, 
                               const Npp8u * pSrc2, int nSrc2Step, 
                                     Npp8u * pDst,  int nDstStep, 
                               NppiSize oSizeROI);

NppStatus nppiAbsDiff_8u_AC4R(const Npp8u * pSrc1, int nSrc1Step, 
                                const Npp8u * pSrc2, int nSrc2Step, 
                                      Npp8u * pDst,  int nDstStep, 
                                NppiSize oSizeROI);



/** 
 * 32bit float, single channel (image) division.
 * Divide pixels in pSrc2 by pSrc1's pixels. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiDiv_32f_C1R(const Npp32f * pSrc1, int nSrc1Step, 
                            const Npp32f * pSrc2, int nSrc2Step, 
                                  Npp32f * pDst,  int nDstStep, 
                            NppiSize oSizeROI);
//@}


/** @name Image Absolute Difference Methods
 * Per-pixel absolute difference methods.
 */
//@{


/** 
 * 8bit unsigned integer, single channel absolute difference.
 * Compute abs(sourcePixel1 - sourcePixel2).
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiAbsDiff_8u_C1R(const Npp8u * pSrc1, int nSrc1Step, 
                               const Npp8u * pSrc2, int nSrc2Step, 
                                     Npp8u * pDst,  int nDstStep, 
                               NppiSize oSizeROI);

/** 
 * 32bit float, single channel absolute difference.
 * Compute abs(sourcePixel1 - sourcePixel2).
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiAbsDiff_32f_C1R(const Npp32f * pSrc1, int nSrc1Step, 
                                const Npp32f * pSrc2, int nSrc2Step, 
                                      Npp32f * pDst,  int nDstStep, 
                                NppiSize oSizeROI);
//@}


/** @name Image Threshold Methods
 * Threshold pixels.
 */
//@{

/** 
 * 32bit float, single channel threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * \param pSrc Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrcStep Number of bytes between line starts of successive lines in first source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param nThreshold The threshold value.
 * \param eComparisonOperation The type of comparison operation to be used for thresholding. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_NOT_SUPPORTED_MODE_ERROR If an invalid comparison operation type is specified.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiThreshold_32f_C1R(const Npp32f * pSrc, int nSrcStep,
                                        Npp32f * pDst, int nDstStep, 
                                  NppiSize oSizeROI, 
                                  Npp32f nThreshold, NppCmpOp eComparisonOperation); 


/** 
 * 8bit unsigned integer, four channel threshold, not affecting alpha channel.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * \param pSrc Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrcStep Number of bytes between line starts of successive lines in first source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param aThresholds The threshold values, one per color channel.
 * \param eComparisonOperation The type of comparison operation to be used for thresholding. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_NOT_SUPPORTED_MODE_ERROR If an invalid comparison operation type is specified.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiThreshold_8u_AC4R(const Npp8u * pSrc, int nSrcStep,
                                        Npp8u * pDst, int nDstStep, 
                                  NppiSize oSizeROI, 
                                  const Npp8u aThresholds[3], NppCmpOp eComparisonOperation);
//@}

/** @name Image Compare Methods
 * Compare the pixels of two images and create a binary result image. In case of multi-channel
 * image types, the condition must be fullfilled for all channels, otherwise the comparison
 * is considered false.
 * The "binary" result image is of type 8u_C1. False is represented by 0, true by NPP_MAX_8U.
 */
//@{

/** 
 * 8bit unsigned integer, four channel (image) compare.
 * Compare pSrc1's pixels with corresponging pixels in pSrc2. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiCompare_8u_C4R(const Npp8u * pSrc1, int nSrc1Step,
                               const Npp8u * pSrc2, int nSrc2Step,
                                     Npp8u * pDst,  int nDstStep,
                               NppiSize oSizeROI, NppCmpOp eComparisonOperation);

/** 
 * 8bit unsigned integer, four channel (image) compare, ignoring alpha channel.
 * Compare pSrc1's pixels with corresponging pixels in pSrc2. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiCompare_8u_AC4R(const Npp8u * pSrc1, int nSrc1Step,
                                const Npp8u * pSrc2, int nSrc2Step,
                                      Npp8u * pDst,  int nDstStep,
                                NppiSize oSizeROI, NppCmpOp eComparisonOperation);

/** 
 * 32bit float, single channel (image) compare.
 * Compare pSrc1's pixels with corresponging pixels in pSrc2. 
 * \param pSrc1 Pointer to the first source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc1Step Number of bytes between line starts of successive lines in first source image..
 * \param pSrc2 Pointer to the second source image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nSrc2Step Number of bytes between line starts of successive lines in second source image..
 * \param pDst Pointer to the destination image's region-of-interest (pixel in upper right-corner of ROI).
 * \param nDstStep Number of bytes between line starts of successive lines in destination image.
 * \param oSizeROI Size (width, height) of the image are to bet set.
 * \param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * \return NPP_NULL_POINTER_ERROR If any of the image pointers are null-pointer.
 * \return NPP_SIZE_ERROR If oSizeROI contains null or negative values.
 * \return NPP_STEP_ERROR If any of the step-sizes is null or negative or if the ROI-width exceeds any 
 *                      of the step-sizes.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 * \return NPP_NO_ERROR If executed sucessfully.
 */
NppStatus nppiCompare_32f_C1R(const Npp32f * pSrc1, int nSrc1Step,
                                const Npp32f * pSrc2, int nSrc2Step,
                                      Npp8u  * pDst,  int nDstStep,
                                NppiSize oSizeROI, NppCmpOp eComparisonOperation);
//@}
 
/** @name Mean_StdDev
 *  Computes the mean and standard deviation of image piexel values
 */
//@{

/**
 * Mean_StdDev 8bit single channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param roiSize Width and Height of the regions of interest.
 * \param pMean Contains computed mean. This is a host pointer.
 * \param pStdDev Contains computed standard deviation. This is a host pointer.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMALLOC_ERR if a CUDA malloc used internally fails
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memset call used internally fails.
 * \return NPP_MEMFREE_ERR if CUDA memfree call used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiMean_StdDev_8u_C1R(const Npp8u * pSrc, int nSrcStep, NppiSize roiSize, Npp64f * pMean, Npp64f * pStdDev );

/** @name NormDiff
 *  Norm of differences between two 8 bit images.
 */
//@{
/**
 * NormDiff_L1 computes the L1 norm of differences between two 8bit single channel images.
 *
 * \param pSrc1 Pointer to the first source image ROI.
 * \param pSrc2 Pointer to the second source image ROI.
 * \param nSrcStep1 Distance in bytes between starts of consecutive lines of the first source image.
 * \param nSrcStep2 Distance in bytes between starts of consecutive lines of the second source image.
 * \param oSizeROI Width and Height in pixels of the source ROI.
 * \param pRetVal Contains computed L1-norm of differences. This is a host pointer.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if one of the specified pointers is NULL.
 * \return NPP_SIZE_ERROR if the ROI has a field with zero or negative value
 * \return NPP_STEP_ERROR if nSrcStep1 or nSrcStep2 is less than oSourceROI * sizeof(Npp8u)
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMALLOC_ERR if a CUDA malloc used internally fails
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memset call used internally fails.
 * \return NPP_MEMFREE_ERR if CUDA memfree call used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiNormDiff_L1_8u_C1R(const Npp8u * pSrc1, int nSrcStep1, 
                                   const Npp8u * pSrc2, int nSrcStep2, 
                                   NppiSize oSizeROI, Npp64f * pRetVal);

/**
 * NormDiff_L2 computes the L2 norm of differences between two 8bit single channel images.
 *
 * \param pSrc1 Pointer to the first source image ROI.
 * \param pSrc2 Pointer to the second source image ROI.
 * \param nSrcStep1 Distance in bytes between starts of consecutive lines of the first source image.
 * \param nSrcStep2 Distance in bytes between starts of consecutive lines of the second source image.
 * \param oSizeROI Width and Height in pixels of the source ROI.
 * \param pRetVal Contains computed L1-norm of differences. This is a host pointer.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if one of the specified pointers is NULL.
 * \return NPP_SIZE_ERROR if the ROI has a field with zero or negative value
 * \return NPP_STEP_ERROR if nSrcStep1 or nSrcStep2 is less than oSourceROI * sizeof(Npp8u)
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMALLOC_ERR if a CUDA malloc used internally fails
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memset call used internally fails.
 * \return NPP_MEMFREE_ERR if CUDA memfree call used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiNormDiff_L2_8u_C1R(const Npp8u * pSrc1, int nSrcStep1, 
                                   const Npp8u * pSrc2, int nSrcStep2, 
                                   NppiSize oSizeROI, Npp64f * pRetVal);

/**
 * NormDiff_L_Inf computes the L_Inf norm of differences between two 8bit single channel images.
 *
 * \param pSrc1 Pointer to the first source image ROI.
 * \param pSrc2 Pointer to the second source image ROI.
 * \param nSrcStep1 Distance in bytes between starts of consecutive lines of the first source image.
 * \param nSrcStep2 Distance in bytes between starts of consecutive lines of the second source image.
 * \param oSizeROI Width and Height in pixels of the source ROI.
 * \param *pRetVal Contains computed L1-norm of differences. This is a host pointer.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if one of the specified pointers is NULL.
 * \return NPP_SIZE_ERROR if the ROI has a field with zero or negative value
 * \return NPP_STEP_ERROR if nSrcStep1 or nSrcStep2 is less than oSourceROI * sizeof(Npp8u)
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMALLOC_ERR if a CUDA malloc used internally fails
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memset call used internally fails.
 * \return NPP_MEMFREE_ERR if CUDA memfree call used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiNormDiff_Inf_8u_C1R(const Npp8u * pSrc1, int nSrcStep1, 
                                    const Npp8u * pSrc2, int nSrcStep2, 
                                    NppiSize oSizeROI, Npp64f * pRetVal);

/** @name 1D Linear Filter
 *  1D mask Linear Convolution Filter, with rescaling, for 8 bit images.
 */
//@{
/**
 * Apply convolution filter with user specified weights for 1D column wise mask.  
 * Result pixel is equal to the sum of the products between the kernel coefficients (pKernel array) and corresponding neighboring 
 * column pixel values in the source image defined by iKernelDim and iAnchorY, divided by iDivisor. 
 *
 * Note: The pixel-weight multiplication is executed as a 24-bit operation for performance reasons. This means that weights
 *  outside the range of a 16-bit signed integer will likely cause wrap-around errors. 
 *
 * \param pSrc Pointer to the start address of the ROI region of the source image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oROI Width and Height of the regions of interest.
 * \param pKernel Pointer to the start address of the kernel coefficient array.  Coeffcients are expected to be stored in reverse order.
 * \param iKernelDim Length of the linear kernel array (must be either 3, 5 or 7 in present implementation)
 * \param iAnchorY Y offset of the kernel origin frame of reference w.r.t the source pixel (must be centered on iKernelDim, i.e. 0.5 * (iKernelDim - 1) in present implementation)
 * \param iDivisor The factor by which the convolved summation from the Filter operation should be divided.  If equal to the sum of coefficients, this will keep the maximum result value within full scale.
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges 
 * \return NPP_MEMCPY_ERROR if there is an error copying mask coefficients from pKernel to Constant memory
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus 
nppiFilterColumn_8u_C1R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oROI, 
                         const Npp32s * pKernel, Npp32s iKernelDim, Npp32s iAnchorY, Npp32s iDivisor);

NppStatus 
nppiFilterColumn_8u_C4R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oROI, 
                         const Npp32s * pKernel, Npp32s nMaskSize, Npp32s nAnchor, Npp32s nDivisor);

/**
 * Apply general linear Row convolution filter, with rescaling, in a 1D mask region around each source pixel for 1-channel 8 bit/pixel images.  
 * Result pixel is equal to the sum of the products between the kernel coefficients (pKernel array) and corresponding neighboring row pixel values in the source image defined by iKernelDim and iAnchorX, divided by iDivisor. 
 *
 * \param pSrc Pointer to the start address of the ROI region of the source image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oROI Width and Height of the regions of interest.
 * \param pKernel Pointer to the start address of the kernel coefficient array.  Coeffcients are expected to be stored in reverse order.  
 * \param iKernelDim Length of the linear kernel array (must be either 3, 5 or 7 in present implementation)
 * \param iAnchorX X offset of the kernel origin frame of reference w.r.t the source pixel (must be centered on iKernelDim, i.e. 0.5 * (iKernelDim - 1) in present implementation)
 * \param iDivisor The factor by which the convolved summation from the Filter operation should be divided.  If equal to the sum of coefficients, this will keep the maximum result value within full scale.
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges 
 * \return NPP_MEMCPY_ERROR if there is an error copying mask coefficients from pKernel to Constant memory
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus 
nppiFilterRow_8u_C1R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oROI, 
                      const Npp32s * pKernel, Npp32s iKernelDim, Npp32s iAnchorX, Npp32s iDivisor);

NppStatus
nppiFilterRow_8u_C4R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oROI, 
                      const Npp32s * pKernel, Npp32s nMaskSize, Npp32s nAnchor, Npp32s nDivisor);

//@}

/** @name 1D Window Sum
 *  1D mask Window Sum for 8 bit images.
 */
//@{
/**
 * Apply Column Window Summation filter over a 1D mask region around each source pixel for 1-channel 8 bit/pixel input images with 32 bit float output.  
 * Result 32 bit float pixel is equal to the sum of the corresponding and neighboring column pixel values in a mask region of the source image defined by iKernelDim and iAnchorY. 
 *
 * \param pSrc Pointer to the start address of the ROI region of the source 8 bit image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination 32 bit float image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oROI Width and Height of the regions of interest.
 * \param nMaskSize Length of the linear kernel array (must be either 3, 5 or 7 in present implementation)
 * \param nAnchor Y offset of the kernel origin frame of reference w.r.t the source pixel (must be centered on iKernelDim, i.e. 0.5 * (iKernelDim - 1) in present implementation)
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges 
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus nppiSumWindowColumn_8u32f_C1R(const Npp8u * pSrc, Npp32s nSrcStep, 
                                          Npp32f * pDst, Npp32s nDstStep, NppiSize oROI, 
                                          Npp32s nMaskSize, Npp32s nAnchor);

/**
 * Apply Row Window Summation filter over a 1D mask region around each source pixel for 1-channel 8 bit/pixel input images with 32 bit float output.  
 * Result 32 bit float pixel is equal to the sum of the corresponding and neighboring row pixel values in a mask region of the source image defined by iKernelDim and iAnchorX. 
 *
 * \param pSrc Pointer to the start address of the ROI region of the source 8 bit image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination 32 bit float image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oROI Width and Height of the regions of interest.
 * \param iMaskDim Length of the linear kernel array (must be either 3, 5 or 7 in present implementation)
 * \param iAnchorX X offset of the kernel origin frame of reference w.r.t the source pixel (must be centered on iKernelDim, i.e. 0.5 * (iKernelDim - 1) in present implementation)
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges 
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus nppiSumWindowRow_8u32f_C1R(const Npp8u * pSrc, Npp32s nSrcStep, Npp32f* pDst, Npp32s nDstStep, NppiSize oROI, Npp32s iMaskDim, Npp32s iAnchorX);
//@}

/** @name 2D Morphology Filter
 *  2D masked Dilate/Erode for 8 bit images.
 */
//@{
/**
 * Apply masked Dilation filter in a 2D mask region around each source pixel for 1-channel 8 bit/pixel images.  
 * Result pixel is equal to the maximum neighboring pixel values within a mask region defined by oMaskSize and oAnchorPoint, but only neighboring pixel values whose corresponding mask values are non-zero are considered.
 *
 * \param pSrc Pointer to the start address of the ROI region of the source image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oSizeROI Width and Height of the regions of interest.
 * \param pMask Pointer to the start address of the mask array
 * \param oMaskSize Width and Height mask array (must be equal to each other and either a 3, 5 or 7 in present implementation)
 * \param oAnchor X and Y offsets of the mask origin frame of reference w.r.t the source pixel (must be centered on oMaskDim, i.e. 0.5 * (oMaskDim.width - 1) in present implementation)
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges, or if none of the coefficients in the array at pMask are non-zero 
 * \return NPP_MEMCPY_ERROR if there is an error copying mask coefficients from pMask to Constant memory
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus 
nppiDilate_8u_C1R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                   const Npp8u * pMask, NppiSize oMaskSize, NppiPoint oAnchor);

NppStatus
nppiDilate_8u_C4R(const Npp8u * pSrc, int nSrcStep, Npp8u * pDst, int nDstStep, NppiSize oSizeROI,
                   const Npp8u * pMask, NppiSize oMaskSize, NppiPoint oAnchor);


/**
 * Apply masked Erosion filter in a 2D mask region around each source pixel for 1-channel 8 bit/pixel images.  
 * Result pixel is equal to the minimum neighboring pixel values within a mask region defined by oMaskSize and oAnchorPoint, but only neighboring pixel values whose corresponding mask values are non-zero are considered.
 *
 * \param pSrc Pointer to the start address of the ROI region of the source image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oSizeROI Width and Height of the regions of interest.
 * \param pMask Pointer to the start address of the mask array
 * \param oMaskSize Width and Height mask array (must be equal to each other and either a 3, 5 or 7 in present implementation)
 * \param oAnchor X and Y offsets of the mask origin frame of reference w.r.t the source pixel (must be centered on oMaskDim, i.e. 0.5 * (oMaskDim.width - 1) in present implementation)
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges, or if none of the coefficients in the array at pMask are non-zero 
 * \return NPP_MEMCPY_ERROR if there is an error copying mask coefficients from pMask to Constant memory
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus 
nppiErode_8u_C1R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                  const Npp8u * pMask, NppiSize oMaskSize, NppiPoint oAnchor);

NppStatus
nppiErode_8u_C4R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI,
                  const Npp8u * pMask, NppiSize oMaskSize, NppiPoint oAnchor);

//@}

/** @name 2D Linear Variable Filters
 *  2D linear filters with variable mask kernels for 8 bit images.
 */
//@{
/**
 * Apply general linear 2D convolution filter, with scaling, for 1-channel 8 bit/pixel images.  
 * Result pixel is equal to the sum of the products between the kernel coefficients (pKernel array) and corresponding neighborhood pixel values in the source image defined by oKernelSize and oAnchor, divided by iDivisor. 
 *
 * \param pSrc Pointer to the start address of the ROI region of the source image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oSizeROI Width and Height of the regions of interest.
 * \param pKernel Pointer to the start address of the kernel coefficient array.  Coeffcients are expected to be stored in reverse order.
 * \param oKernelSize Width and Height of the rectangular kernel (must be either 3, 5 or 7 and equal to each other in present implementation)
 * \param oAnchor X and Y offsets of the kernel origin frame of reference w.r.t the source pixel (must be centered, i.e. 0.5 * (width - 1) in present implementation)
 * \param nDivisor The factor by which the convolved summation from the Filter operation should be divided.  If equal to the sum of coefficients, this will keep the maximum result value within full scale.
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges
 * \return NPP_MEMCPY_ERROR if there is an error copying mask coefficients from pMask to Constant memory
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus 
nppiFilter_8u_C1R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                   const Npp32s * pKernel, NppiSize oKernelSize, NppiPoint oAnchor, Npp32s nDivisor);
                   
NppStatus 
nppiFilter_8u_C4R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                   const Npp32s * pKernel, NppiSize oKernelSize, NppiPoint oAnchor, Npp32s nDivisor);

//@}

/** @name 2D Linear Fixed Filters
 *  2D linear fixed filters for 8 bit images.
 */
//@{
/**
 * Find average local pixel value in a 2D mask region around each source pixel for 1-channel 8bit/pixel images.
 *
 * \param pSrc Pointer to the start address of the ROI region of the source image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oSizeROI Width and Height of the regions of interest.
 * \param oMaskSize Width and Height of the neighborhood region for the local Avg operation (must be either 3, 5 or 7 and equal to each other in present implementation)
 * \param oAnchor X and Y offsets of the kernel origin frame of reference w.r.t the source pixel (must be centered, i.e. 0.5 * (width - 1) in present implementation)
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus 
nppiFilterBox_8u_C1R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                      NppiSize oMaskSize, NppiPoint oAnchor);

NppStatus 
nppiFilterBox_8u_C4R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                      NppiSize oMaskSize, NppiPoint oAnchor);

//@}

/** @name 2D Tile Max/Median/Min
 *  2D mask Max/Median/Min for 8 bit images.
 */
//@{
/**
 * Find Maximum local pixel value in a 2D mask region around each source pixel for 1-channel 8bit/pixel images.
 *
 * \param pSrc Pointer to the start address of the ROI region of the source image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oSizeROI Width and Height of the regions of interest.
 * \param oMaskSize Width and Height of the neighborhood region for the local Max operation (must be either 3, 5 or 7 and equal to each other in present implementation)
 * \param oAnchor X and Y offsets of the kernel origin frame of reference w.r.t the source pixel (must be centered, i.e. 0.5 * (width - 1) in present implementation)
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus 
nppiFilterMax_8u_C1R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                      NppiSize oMaskSize, NppiPoint oAnchor);

NppStatus
nppiFilterMax_8u_C4R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                      NppiSize oMaskSize, NppiPoint oAnchor);

/**
 * Find Minimum local pixel value in a 2D mask region around each source pixel for 1-channel 8bit/pixel images.
 *
 * \param pSrc Pointer to the start address of the ROI region of the source image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param pDst Pointer to the start address of the target ROI region in the destination image (offset from the head address of the image, if ROI is displaced from image boundary)
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.  Frequently is NOT equal to image width in pixels due to padding for alignment or mem access pattern optimization.
 * \param oSizeROI Width and Height of the regions of interest.
 * \param oMaskSize Width and Height of the neighborhood region for the local Min operation (must be either 3, 5 or 7 and equal to each other in present implementation)
 * \param oAnchor X and Y offsets of the kernel origin frame of reference w.r.t the source pixel (must be centered, i.e. 0.5 * (width - 1) in present implementation)
 *
 * \return NPP_SUCCESS For successful completion.
 * \return NPP_NULL_POINTER_ERROR if pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR if oROI Width or Height is Negative 
 * \return NPP_STEP_ERROR if nSrcStep or nDstStep is less than oROI.Width
 * \return NPP_INVALID_INPUT if supplied function arguments are outside currently supported ranges
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if there is an error in the computational kernel launch
 */
NppStatus 
nppiFilterMin_8u_C1R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                      NppiSize oMaskSize, NppiPoint oAnchor);

NppStatus 
nppiFilterMin_8u_C4R(const Npp8u * pSrc, Npp32s nSrcStep, Npp8u * pDst, Npp32s nDstStep, NppiSize oSizeROI, 
                      NppiSize oMaskSize, NppiPoint oAnchor);

//@}

/**
 * Computes histogram of 8bit single channel image using equal bins.
 *
 * \param pSrc Pointer to the source image ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param pHist Pointer to the computed histogram.
 * \param pLevels Pointer to array of level values.
 * \param nLevels Number of levels
 * \param nLowerLevel Lower level boundary
 * \param nUpperLevel Upper level boundary

 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if one of the specified pointers is NULL.
 * \return NPP_SIZE_ERROR indicates an error if oROI has a field with zero or negative value.
 * \return NPP_MEM_ALLOC_ERR indicates an error if there is not enough memory for the inner histogram.
 * \return NPP_HISTO_NUMBER_OF_LEVELS_ERROR indicates an error if nLevels is less than 2.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if the CUDA kernel fails.
 */
NppStatus nppiHistogramEven_8u_C1R(const Npp8u * pSrc, int nSrcStep, NppiSize oSizeROI, Npp32s * pHist, Npp32s * pLevels,
                                     int nLevels, Npp32s nLowerLevel, Npp32s nUpperLevel);

/**
 * Computes histogram of 8bit four channel image using equal bins.
 *
 * \param pSrc Pointer to the source image ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param pHist Pointer to the computed histogram. An array of pointers to the histogram for each channel.
 * \param pLevels An array of pointers to the level values array for each channel.
 * \param nLevels Number of levels, separate for each channel.
 * \param nLowerLevel Lower level boundary, separate for each channel.
 * \param nUpperLevel Upper level boundary, separate for each channel.

 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if one of the specified pointers is NULL.
 * \return NPP_SIZE_ERROR indicates an error if oROI has a field with zero or negative value.
 * \return NPP_MEM_ALLOC_ERR indicates an error if there is not enough memory for the inner histogram.
 * \return NPP_HISTO_NUMBER_OF_LEVELS_ERROR indicates an error if nLevels is less than 2.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if the CUDA kernel fails.
 */
NppStatus nppiHistogramEven_8u_C4R(const Npp8u * pSrc, int nSrcStep, NppiSize oSizeROI, Npp32s* pHist[4], Npp32s* pLevels[4],
                                     int nLevels[4], Npp32s nLowerLevel[4], Npp32s nUpperLevel[4]);

/** @name JPEG DCT, Quantization and Level Shift Functions
 *  Jpeg standard defines a flow of level shift, DCT and quantization for forward JPEG transform
 *  and inverse level shift, IDCT and de-quantization for inverse JPEG transform. This group 
 *  has the functions for both forward and inverse functions. 
 */
//@{

/**
 * Converts regular quantization tables with the quality factor.
 *
 * \param pQuantRawTable Raw quantization table.
 * \param nQualityFactor Quality factor for the table. Range is [1:100].
 *
 * \return NPP_NULL_POINTER_ERROR pQuantRawTable is a null pointer.
 * \return NPP_SUCCESS For successful completetion.
 */
NppStatus nppiQuantFwdRawTableInit_JPEG_8u(Npp8u* pQuantRawTable, int nQualityFactor);

/**
 * Converts raw quantization table to a forward quantization table.
 *
 * \param pQuantRawTable Raw quantization table.
 * \param pQuantFwdRawTable Forward quantization table.
 *
 * \return NPP_NULL_POINTER_ERROR pQuantRawTable is a null pointer.
 * \return NPP_SUCCESS For successful completetion.
 */
NppStatus nppiQuantFwdTableInit_JPEG_8u16u(const Npp8u* pQuantRawTable, Npp16u* pQuantFwdRawTable);

/**
 * Converts raw quantization table to an inverse quantization table.
 *
 * \param pQuantRawTable Raw quantization table.
 * \param pQuantFwdRawTable Inverse quantization table.
 *
 * \return NPP_NULL_POINTER_ERROR pQuantRawTable or pQuantFwdRawTable is a null pointer.
 * \return NPP_SUCCESS For successful completetion.
 */
NppStatus nppiQuantInvTableInit_JPEG_8u16u(const Npp8u * pQuantRawTable, Npp16u * pQuantFwdRawTable);

/**
 * Fills out the quantization table with either luminance and chrominance tables for JPEG.
 *
 * \param pQuantRawTable Raw quantization table.
 * \param tableIndex Choice for Luminance (tableIndex is 0) or Chrominance component (tableIndex is 1).
 *
 * \return NPP_NULL_POINTER_ERROR pQuantRawTable is a null pointer.
 * \return NPP_INVALID_INPUT tableIndex is not 0 or 1.
 * \return NPP_SUCCESS For successful completetion.
 */
NppStatus nppiSetDefaultQuantTable ( Npp8u* pQuantRawTable, int tableIndex );

/**
 * Inverse DCT, de-quantization and level shift part of the JPEG decoding.
 * Input is expected in 64x1 macroblocks and output is expected to be in 8x8 macroblocks.
 *
 * \param pSrc Pointer to the 16-bit source image.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the 8-bit destination image.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param pQuantInvTable Inverse quantization tables for JPEG decoding.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_SIZE_ERROR For negative input height/width or not a multiple of 8 width/height.
 * \return NPP_STEP_ERROR If input image width is not multiple of 8 or does not match ROI.
 * \return NPP_NULL_POINTER_ERROR If the destination pointer is NULL.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If one of the CUDA calls caused an error, this is returned.
 */
NppStatus nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R(Npp16s *pSrc, int srcStep, Npp8u *pDst, int dstStep, const Npp16u* pQuantInvTable, NppiSize oSizeROI);

/**
 * Forward DCT, quantization and level shift part of the JPEG encoding.
 * Input is expected in 8x8 macroblocks and output is expected to be in 64x1 macroblocks.
 *
 * \param pSrc Pointer to the 8-bit source image.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the 16-bit destination image.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param pQuantFwdTable Forward quantization tables for JPEG encoding.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_SIZE_ERROR For negative input height/width or not a multiple of 8 width/height.
 * \return NPP_STEP_ERROR If input image width is not multiple of 8 or does not match ROI.
 * \return NPP_NULL_POINTER_ERROR If the destination pointer is NULL.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If one of the CUDA calls caused an error, this is returned.
 */
NppStatus nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R(Npp8u *pSrc, int srcStep, Npp16s *pDst,  int dstStep, const Npp16u* pQuantFwdTable, NppiSize oSizeROI);

/**
 * SqrIntegral Transforms an image to integral and integral of pixel squares representation. This function assumes that the integral and integral of squares 
 * images 

 *
 * \param pSrc Pointer to the source ROI.
 * \param nSrcStep Distance in bytes between start of consecutive lines of the source image.
 * \param pDst Pointer to the ROI in the destination integral image.
 * \param nDstStep Distance in bytes between start of consecutive lines in the destination  integral image.
 * \param pSqr Pointer to the ROI in the destination integral image of pixel squares.
 * \param nSqrStep Distance in bytes between start of consecutive lines in the destination square integral image 
 * \param srcROI Size of source image ROI in pixels (width and height).
 * \param val The value to add to pDst image pixels
 * \param valSqr The value to add to pSqr image pixels
 * \param integralImageNewHeight Extended height of output surfacess (needed by transpose in primitive)

 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if one of the pointers is NULL.
 * \return NPP_SIZE_ERROR if a source ROI field has 0 or negative value.
 * \return NPP_STEP_ERROR if nSrcStep is less than srcROI.width*sizeof(Npp8u) or nDstStep is less than nDstStep< (int)((srcROI.width + 1)*sizeof(Npp32s)) 
 * or if nSqrStep < (int)((srcROI.width + 1)*sizeof(Npp32f)).
 * \return NPP_NOT_EVEN_STEP_ERROR indicates an error condition if nDstStep or nSqrStep is not divisible by 4.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if a kernel or device operation fails.
 */
NppStatus nppiSqrIntegral_8u32s32f_C1R(Npp8u *pSrc, int nSrcStep, 
                                         Npp32s *pDst, int nDstStep, Npp32f *pSqr, int nSqrStep,
                                         NppiSize srcROI, Npp32s val, Npp32f valSqr, Npp32s integralImageNewHeight);

//@}

/**
 * RectStdDev Computes the standard deviation of itnegral images
 *
 * \param pSrc Pointer to the source integral image.
 * \param nSrcStep Distance in bytes between start of consecutive lines in the source integral image.
 * \param pSqr Pointer to the ROI in the source integral image of pixel squares.
 * \param nSqrStep Distance in bytes between start of consecutive lines in the source integral image of pixel squares 
 * \param pDst Pointer to the destination image ROI.
 * \param nDstStep Distance in bytes between start of consecutive lines in the destination  image.
 * \param roiSize Size of destination image ROI in pixels (width and height).
 * \param rect rectangular window

 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR if roiSize has a field with zero or negative value
 * \return NPP_SIZE_ERROR if rect.width or rect.height is less than or equal to zero or if rect.x or rect.y is less than zero.
 * \return NPP_STEP_ERROR if nSrcStep is less than (roiSize.width + rect.x + rect.width + 1)* sizeof(Npp32s) or if nSqrStep is less than 
 * (roiSize.width + rect.x + rect.width + 1)* sizeof(Npp32f) or if nDstStep is less than roiSize.width * sizeof(Npp32f).
 * \return NPP_NOT_EVEN_STEP_ERROR if either nSrcStep, nSqrStep or nDstStep is not divisible by 4.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR if the CUDA kernel fails.
 * \return NPP_TEXTURE_BIND_ERROR if a CUDA texture bind or unbind operation fails.
 */
//@}
NppStatus nppiRectStdDev_32s32f_C1R(const Npp32s *pSrc, int nSrcStep, const Npp32f *pSqr, int nSqrStep, 
                                      Npp32f *pDst, int nDstStep, NppiSize roiSize, NppiRect rect);

/** @name Sum
 *  Sum of 8 bit images.
 */
//@{

/**
 * Sum 8bit single channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param oROI Width and height of the regions of interest.
 * \param *pSum Contains computed sum. This is a host pointer.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMALLOC_ERR if a CUDA malloc used internally fails
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memset call used internally fails.
 * \return NPP_MEMFREE_ERR if CUDA memfree call used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiSum_8u_C1R(const Npp8u *pSrc, int nSrcStep, NppiSize oROI, 
                           Npp64f *pSum);

/**
 * Sum 8bit four channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param oROI Width and height of the regions of interest.
 * \param aSum Array contains computed sum for each channel. This is a host pointer.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMALLOC_ERR if a CUDA malloc used internally fails
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memset call used internally fails.
 * \return NPP_MEMFREE_ERR if CUDA memfree call used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 * \return NPP_STEP_ERROR if nSrcStep is not a multiple of 4
 */
NppStatus nppiSum_8u_C4R(const Npp8u *pSrc, int nSrcStep, NppiSize oROI, 
                           Npp64f aSum[4]);
//@}
/** @name Minmax
 *  Minimum and maximum of 8bit images.
 */
//@{
/**
 * Minimum and Maximum of 8bit single channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param oROI Width and height of the regions of interest.
 * \param *pMin Contains computed minimum.
 * \param *pMax Contains computed maximum.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pMin, pMax or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMALLOC_ERR if a CUDA malloc used internally fails
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memset call used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memfree call used internally fails.
 */
NppStatus nppiMinMax_8u_C1R(const Npp8u *pSrc, int nSrcStep, NppiSize oROI, 
                           Npp8u *pMin, Npp8u *pMax);

/**
 * Minimum and Maximum of 8bit four channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param oROI Width and height of the regions of interest.
 * \param cuMin Array contains computed minimum channel values.
 * \param cuMax Array contains computed maximum channel values.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pMin, pMax or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMALLOC_ERR if a CUDA malloc used internally fails
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memset call used internally fails.
 * \return NPP_MEMSET_ERR if CUDA memfree call used internally fails.
 */
NppStatus nppiMinMax_8u_C4R(const Npp8u *pSrc, int nSrcStep, NppiSize oROI, 
                           Npp8u cuMin[4], Npp8u cuMax[4]);
//@}

/** @name Resize
 *  Resizes 8 bit images. Handles C1 and C4 images.
 */
//@{
/**
 * Resize 8bit single channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcSize Size in pixels of the source image
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param srcROI Region of interest in the source image.
 * \param pDst Pointer to the destination ROI.
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param dstROISize Size in pixels of the destination image
 * \param xFactor Factors by which x dimension is changed 
 * \param yFactor Factors by which y dimension is changed 
 * \param interpolation The type of interpolation to perform resampling
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR indicates an error condition if eitehr pSrc or pDst pointer is NULL
 * \return NPP_SIZE_ERROR indicates an error condition if srcSize or dstRoiSize or srcROI has a field with zero or negative value.
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR indicates an error condition if srcROIRect has no intersection with the source image.
 * \return NPP_RESIZE_NO_OPERATION_ERROR if either destination ROI width or height is less than 1 pixel.
 * \return NPP_RESIZE_FACTOR Indicates an error condition if eitehr xFactor or yFactor is less than or equal to zero.
 * \return NPP_INTERPOLATION_ERROR if interpolation has an illegal value.
 * \return NPP_KERNEL_EXECUTION_ERROR return value in case of a CUDA kernel failure.
 */
NppStatus nppiResize_8u_C1R(const Npp8u *pSrc, NppiSize srcSize, int nSrcStep, 
                              NppiRect srcROI, Npp8u *pDst, int nDstStep, NppiSize dstROISize,
                              double xFactor, double yFactor, int interpolation);
/**
 * Resize 8bit four channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcSize Size in pixels of the source image
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param srcROI Region of interest in the source image.
 * \param pDst Pointer to the destination ROI.
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param dstROISize Size in pixels of the destination image
 * \param xFactor Factors by which x dimension is changed 
 * \param yFactor Factors by which y dimension is changed 
 * \param interpolation The type of interpolation to perform resampling
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR indicates an error condition if eitehr pSrc or pDst pointer is NULL
 * \return NPP_SIZE_ERROR indicates an error condition if srcSize or dstRoiSize or srcROI has a field with zero or negative value.
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR indicates an error condition if srcROIRect has no intersection with the source image.
 * \return NPP_RESIZE_NO_OPERATION_ERROR if either destination ROI width or height is less than 1 pixel.
 * \return NPP_RESIZE_FACTOR Indicates an error condition if eitehr xFactor or yFactor is less than or equal to zero.
 * \return NPP_INTERPOLATION_ERROR if interpolation has an illegal value.
 * \return NPP_KERNEL_EXECUTION_ERROR return value in case of a CUDA kernel failure.

 */
NppStatus nppiResize_8u_C4R(const Npp8u *pSrc, NppiSize srcSize, int nSrcStep, 
                              NppiRect srcROI, Npp8u *pDst, int nDstStep, NppiSize dstROISize,
                              double xFactor, double yFactor, int interpolation);

//@}

/** @name Rotate
 *  Rotates an image around the origin (0,0) and then shifts it.
 */
//@{
/**
 * Rotate 8bit single channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcSize Size in pixels of the source image
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param srcROI Region of interest in the source image.
 * \param pDst Pointer to the destination ROI.
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param dstROI Region of interest in the destination image.
 * \param angle The angle of rotation in degrees.
 * \param xShift Shift along horizontal axis 
 * \param yShift Shift along vertical axis 
 * \param interpolation The type of interpolation to perform resampling
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR indicates an error condition if either pSrc or pDst pointer is NULL.
.*.\return NPP_SIZE_ERROR indicates an error condition if any image dimension has zero or negative value.
 * \return NPP_STEP_ERROR indicates an error if nSrcStep or nDstStep has zero or negative value.
 * \return NPP_INTERPOLATION_ERROR if interpolation has an illegal value.
 * \return NPP_RECT_ERROR Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1.
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR indicates an error condition if srcROIRect has no intersection with the source image.
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING indicates a warning that no operation is performed if the transformed source ROI does not intersect the destination ROI.
 * \return NPP_KERNEL_EXECUTION_ERROR return value in case of a CUDA kernel failure.
 */
NppStatus nppiRotate_8u_C1R(const Npp8u *pSrc, NppiSize srcSize, int nSrcStep,
                              NppiRect srcROI, Npp8u *pDst, int nDstStep, NppiRect dstROI,
                              double angle, double xShift, double yShift, int interpolation);
/**
 * Rotate 8bit four channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcSize Size in pixels of the source image
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param srcROI Region of interest in the source image.
 * \param pDst Pointer to the destination ROI.
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param dstROI Region of interest in the destination image.
 * \param angle The angle of rotation in degrees.
 * \param xShift Shift along horizontal axis 
 * \param yShift Shift along vertical axis 
 * \param interpolation The type of interpolation to perform resampling
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR indicates an error condition if either pSrc or pDst pointer is NULL.
.*.\return NPP_SIZE_ERROR indicates an error condition if any image dimension has zero or negative value.
 * \return NPP_STEP_ERROR indicates an error if nSrcStep or nDstStep has zero or negative value.
 * \return NPP_INTERPOLATION_ERROR if interpolation has an illegal value.
 * \return NPP_RECT_ERROR Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1.
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR indicates an error condition if srcROIRect has no intersection with the source image.
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING indicates a warning that no operation is performed if the transformed source ROI does not intersect the destination ROI.
 * \return NPP_KERNEL_EXECUTION_ERROR return value in case of a CUDA kernel failure.
 */
NppStatus nppiRotate_8u_C4R(const Npp8u *pSrc, NppiSize srcSize, int nSrcStep,
                              NppiRect srcROI, Npp8u *pDst, int nDstStep, NppiRect dstROI,
                              double angle, double xShift, double yShift, int interpolation);
//@}
/** @name Mirror
 *  Mirrors 8bit images about either the horizontal, vertical or both axis.
 */
//@{
/**
 * Mirror 8bit single channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oROI Width and height of the regions of interest.
 * \param flip Specifies the axis about which the image is to be mirrored.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if either pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR For negative input height/width.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                              of the step-sizes or if nSrcStep or nDstStep are not 4-byte aligned.
 *.\return NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * \return NPP_TEXTURE_BIND_ERROR if either CUDA texture bind or unbind API call fails.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 */
NppStatus nppiMirror_8u_C1R(const Npp8u * pSrc, int nSrcStep, 
                              Npp8u * pDst, int nDstStep, 
                              NppiSize oROI, NppiAxis flip);
/**
 * Mirror 8bit four channel image.
 *
 * \param pSrc Pointer to the source ROI.
 * \param nSrcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param nDstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oROI Width and height of the regions of interest.
 * \param flip Specifies the axis about which the image is to be mirrored.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if either pSrc or pDst is NULL
 * \return NPP_SIZE_ERROR For negative input height/width.
 * \return NPP_STEP_ERROR If nSrcStep or nDstStep is null or negative or if the ROI-width exceeds on
 *                              of the step-sizes or if nSrcStep or nDstStep are not 4-byte aligned.
 *.\return NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * \return NPP_TEXTURE_BIND_ERROR if either CUDA texture bind or unbind API call fails.
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR If CUDA kernel could not be executed.
 */
NppStatus nppiMirror_8u_C4R(const Npp8u * pSrc, int nSrcStep, 
                              Npp8u * pDst, int nDstStep, 
                              NppiSize oROI, NppiAxis flip);
//@}

//@}
/** @name RGBToYCbCr 
 *  RGB to YCbCr color conversion.
 */
//@{
/**
 * RGB to YCbCr color conversion for 3-channel pixel-packed 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiRGBToYCbCr_8u_C3R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI);

/**
 * RGB to YCbCr422 (2-channel pixel-packed) color conversion for 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiRGBToYCbCr422_8u_C3C2R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI);

/**
 * RGB to YCbCr420 (3-plane) color conversion for 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiRGBToYCbCr420_8u_C3P3R(const Npp8u * pSrc, int srcStep, Npp8u ** pDst, int dstStep[3], NppiSize oSizeROI);

/**
 * RGB to YCbCr color conversion for 3-channel planar 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiRGBToYCbCr_8u_P3R(const Npp8u * const * pSrc, int srcStep, Npp8u ** pDst, int dstStep, NppiSize oSizeROI);

/**
 * RGB to YCbCr color conversion for 4-channel pixel-packed 8bit/pixel images. 
 * Alpha channel is the last channel and is not processed.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiRGBToYCbCr_8u_AC4R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI);

//@}

//@}
/** @name YCbCrToRGB 
 *  YCbCr to RGB color conversion.
 */
//@{
/**
 * YCbCr to RGB color conversion for 3-channel pixel-packed 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiYCbCrToRGB_8u_C3R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI);

/**
 * YCbCr to RGB color conversion for 3-channel planar 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiYCbCrToRGB_8u_P3R(const Npp8u * const * pSrc, int srcStep, Npp8u ** pDst, int dstStep, NppiSize oSizeROI);

/**
 * YCbCr to RGB color conversion for 4-channel pixel-packed 8bit/pixel images.
 * Alpha channel is the last channel and is not processed.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiYCbCrToRGB_8u_AC4R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI);

/**
 * YCbCr422 (2-channel pixel-packed) to RGB color conversion for 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiYCbCr422ToRGB_8u_C2C3R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI); 

/**
 * YCbCr420 (3-plane) to RGB color conversion for 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiYCbCr420ToRGB_8u_P3C3R(const Npp8u * const * pSrc, int srcStep[3], Npp8u * pDst, int dstStep, NppiSize oSizeROI);

//@}


//@}
/** @name YCbCr422ToYCbCr420 
 *  YCbCr to YCbCr spatial sampling conversion.
 */
//@{

/**
 * YCbCr:422 to YCbCr:420 spatial sampling conversion for 3-channel planar 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiYCbCr422ToYCbCr420_8u_P3R(const Npp8u * const * pSrc, int srcStep[3], Npp8u ** pDst, int dstStep[3], NppiSize oSizeROI);

//@}

//@}
/** @name YCbCr422ToYCbCr411 
 *  YCbCr to YCbCr spatial sampling conversion.
 */
//@{

/**
 * YCbCr:422 to YCbCr:411 spatial sampling conversion for 3-channel planar 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiYCbCr422ToYCbCr411_8u_P3R(const Npp8u * const * pSrc, int srcStep[3], Npp8u ** pDst, int dstStep[3], NppiSize oSizeROI);

//@}
/** @name YCbCr420ToYCbCr422 
 *  YCbCr to YCbCr spatial sampling conversion.
 */
//@{

/**
 * YCbCr:420 to YCbCr:422 spatial sampling conversion for 3-channel planar 8bit/pixel images.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiYCbCr420ToYCbCr422_8u_P3R(const Npp8u * const * pSrc, int srcStep[3], Npp8u ** pDst, int dstStep[3], NppiSize oSizeROI);

//@}

//@}
/** @name YCbCr420ToYCbCr411 
 *  YCbCr to YCbCr spatial sampling conversion.
 */
//@{

/**
 * YCbCr:420 to YCbCr:411 spatial sampling conversion for 3-planes in 2-planes out planar 8bit/pixel images.
 *
 * \param pSrc Array of pointers to the source image planes.
 * \param aSrcStep Array with distances in bytes between starts of consecutive lines of the source image planes.
 * \param pDstY Pointer to the destination ROI of Y-channel.
 * \param nDstYStep Distance in bytes between starts of consecutive lines of the destination Y-channel.
 * \param pDstCbCr Pointer to the destination ROI of CbCr destination image.
 * \param nDstCbCrStep Distance in bytes between starts of consecutive lines of the interleaved CbCr destination image.
 * \param oSizeROI Width and height of the regions of interest.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiYCbCr420ToYCbCr411_8u_P3P2R(const Npp8u * const * pSrc, int aSrcStep[3], 
                                            Npp8u * pDstY, int nDstYStep, Npp8u * pDstCbCr, int nDstCbCrStep, NppiSize oSizeROI);

//@}


//@}
/** @name ColorTwist32f
 *  Color mapping function.
 */
//@{
/**
 * Color mapping functions for 3-channel pixel-packed 8bit/pixel images.
 * An input color twist matrix with floating-point pixel values is applied within ROI.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param twist The color twist matrix with floating-point pixel values.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiColorTwist32f_8u_C3R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI, const Npp32f twist[3][4]);

/**
 * Color mapping functions for 3-channel planar 8bit/pixel images.
 * An input color twist matrix with floating-point pixel values is applied within ROI.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param twist The color twist matrix with floating-point pixel values.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiColorTwist32f_8u_P3R(const Npp8u * const * pSrc, int srcStep, Npp8u ** pDst, int dstStep, NppiSize oSizeROI, const Npp32f twist[3][4]);

/**
 * Color mapping functions for 4-channel pixel-packed 8bit/pixel images.
 * An input color twist matrix with floating-point pixel values is applied within ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param twist The color twist matrix with floating-point pixel values.
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 */
NppStatus nppiColorTwist32f_8u_AC4R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI, const Npp32f twist[3][4]);

//@}

//@}
/** @name LUT_Linear
 *  Look-up table (LUT) mapping function.
 */
//@{
/**
 * Look-up table (LUT) mapping functions for 1-channel 8bit/pixel images.
 * The LUT is derived from a set of user defined mapping points through linear interpolation. 
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param pValues Pointer to an array of user defined OUTPUT values
 * \param pLevels Pointer to an array of user defined INPUT values
 * \param nLevels Number of user defined input/output mapping points (levels)
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 * \return NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2.
 */
NppStatus nppiLUT_Linear_8u_C1R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI, 
                                  const Npp32s * pValues, const Npp32s * pLevels, int nLevels);

/**
 * Look-up table (LUT) mapping functions for 3-channel pixel-packed 8bit/pixel images.
 * The LUT is derived from a set of user defined mapping points through linear interpolation. 
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param pValues Double pointer to an [3] of arrays of user defined OUTPUT values per CHANNEL
 * \param pLevels Double pointer to an [3] of arrays of user defined INPUT values per CHANNEL
 * \param nLevels A [3] array of user defined input/output mapping points (levels) per CHANNEL
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 * \return NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2.
 */
NppStatus nppiLUT_Linear_8u_C3R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI, 
                                  const Npp32s * pValues[3], const Npp32s * pLevels[3], int nLevels[3]);

/**
 * Look-up table (LUT) mapping functions for 4-channel pixel-packed 8bit/pixel images.
 * The LUT is derived from a set of user defined mapping points through linear interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * \param pSrc Pointer to the source ROI.
 * \param srcStep Distance in bytes between starts of consecutive lines of the source image.
 * \param pDst Pointer to the destination ROI.
 * \param dstStep Distance in bytes between starts of consecutive lines of the destination image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param pValues Double pointer to an [4] of arrays of user defined OUTPUT values per CHANNEL
 * \param pLevels Double pointer to an [4] of arrays of user defined INPUT values per CHANNEL
 * \param nLevels A [4] array of user defined input/output mapping points (levels) per CHANNEL
 *
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR if pSum or pSrc pointer NULL.
 * \return NPP_SIZE_ERROR if oROI has a field with zero or negative value
 * \return NPP_KERNEL_EXECUTION_ERROR if CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 * \return NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2.
 */
NppStatus nppiLUT_Linear_8u_AC4R(const Npp8u * pSrc, int srcStep, Npp8u * pDst, int dstStep, NppiSize oSizeROI, 
                                   const Npp32s * pValues[4], const Npp32s * pLevels[4], int nLevels[4]);

//@}

/** 
 * Canny edge detection
 * Find edges using the Canny algorithm.  This function requires a temporary working device buffer
 * whose size should be computed by a prior call to nppiCannyGetSize. 
 * \param pSrcDx Pointer to the source image ROI x-derivative.
 * \param nSrcDxStep Distance in bytes between starts of consecutive lines in the source image pSrcDx.
 * \param pSrcDy Pointer to the source image ROI y-derivative.
 * \param nSrcDyStep Distance in bytes between starts of consecutive lines in the source image pSrcDy
 * \param pDstEdges Pointer to the output array of the detected edges. 
 * \param nDstEdgeStep Distance in bytes between starts of consecutive lines in the output image.
 * \param oSizeROI Width and height of the regions of interest.
 * \param nLowThreshold Lower threshold for edge detection.
 * \param nHighThreshold Upper threshold for edge detection. 
 * \param pBuffer Pointer to the pre-allocated temporary buffer. 
 *
 * \return NPP_SUCCESS For successful completion. 
 * \return NPP_NULL_POINTER_ERROR if any of the pointer arguments is NULL. 
 * \return NPP_SIZE_ERROR if roiSize has a zero or negative value in a field. 
 * \return NPP_STEP_ERROR if any step is less than roiSize.width * 4 
 * \return NPP_BAD_ARG_ERROR if lowThresh is negative of highThresh is less than lowThresh
 * \return NPP_NOT_EVEN_STEP_ERROR if any step size is not divisible by 4.
 * \return NPP_KERNEL_EXECUTION_ERROR if a CUDA kernel could could not be executed properly.
 * \return NPP_MEMCPY_ERROR if a CUDA memory copy (either from host to device or device to host) used internally fails.
 * \return NPP_TEXTURE_BIND_ERROR if either texture bind or unbind call used internally fails.
 * \return NPP_MEM_ALLOC_ERR if internal texture buffer allocation fails
 * \return NPP_WARNING if the edge detection did not converge within the maximum number of iterations
 * 
 */ 
NppStatus nppiCanny_32f8u_C1R(const Npp32f *pSrcDx, int nSrcDxStep,
                                const Npp32f *pSrcDy, int nSrcDyStep,
                                Npp8u *pDstEdges, int nDstEdgeStep,
                                NppiSize oSizeROI,
                                Npp32f nLowThreshold, Npp32f nHighThreshold,
                                Npp8u *pBuffer);

/** @name CannyGetSize
 *  Calculate the size of a temporary buffer needed for the Canny function.
 *
 * \param roiSize Size of the image ROI in pixels
 * \param pBufSize Pointer to the computed size of the temporary buffer
 * 
 * \return NPP_SUCCESS For successful completetion.
 * \return NPP_NULL_POINTER_ERROR indicates an error condition if pBufSize point is NULL
 * \return NPP_SIZE_ERROR if roiSize has a field with zero or negative value
 */
NppStatus nppiCannyGetSize( NppiSize roiSize, int *pSufSize); 



/** @name Affine warping, affine transform calculation
 *  Affine warping of an image is the transform of image pixel positions, defined by the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * C = \left[ \matrix{C_{00} & C_{01} & C_{02} \cr C_{10} & C_{11} & C_{12} } \right]
 * \f]
 * That is, any pixel of the transformed image with coordinates \f$(X_{new},Y_{new})\f$ has a preimage with 
 * coordinates \f$(x,y)\f$. The mapping \f$C\f$ is fully defined by 6 values \f$C_{ij}, i=\overline{0,1}, j=\overline{0,2}\f$.
 * The transform has a properties of mapping any parallel lines to parallel lines and preserving ratios of distances between points on a line.
 * The NPPI implementation of affine transform has some issues which are discussed in each function's documentation.
 */
//@{


/**
 * Calculates affine transform coefficients given source rectangular ROI and its destination quadrangle projection
 *
 * \param srcRoi                 [IN] Source ROI
 * \param quad                   [IN] Destination quadrangle
 * \param coeffs                 [OUT] Affine transform coefficients
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_AFFINE_QUAD_INCORRECT_WARNING        Indicates a warning when quad doesn't conform to the transform properties. Fourth vertex is ignored, internally computed coordinates are used instead
 */
NppStatus nppiGetAffineTransform(NppiRect srcRoi, const double quad[4][2], double coeffs[2][3]);


/**
 * Calculates affine transform projection of given source rectangular ROI
 *
 * \param srcRoi                 [IN] Source ROI
 * \param quad                   [OUT] Destination quadrangle
 * \param coeffs                 [IN] Affine transform coefficients
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 */
NppStatus nppiGetAffineQuad(NppiRect srcRoi, double quad[4][2], const double coeffs[2][3]);


/**
 * Calculates bounding box of the affine transform projection of the given source rectangular ROI
 *
 * \param srcRoi                 [IN] Source ROI
 * \param bound                  [OUT] Bounding box of the transformed source ROI
 * \param coeffs                 [IN] Affine transform coefficients
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 */
NppStatus nppiGetAffineBound(NppiRect srcRoi, double bound[2][2], const double coeffs[2][3]);


/**
 * Affine transform of an image (8bit unsigned integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly. The function operates on source and destination regions 
 * of interest. The affine warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Affine transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpAffine_8u_C1R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                       int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                       int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, three channels).
 * \see nppiWarpAffine_8u_C1R
 */
NppStatus nppiWarpAffine_8u_C3R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                       int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                       int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, four channels).
 * \see nppiWarpAffine_8u_C1R
 */
NppStatus nppiWarpAffine_8u_C4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                       int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                       int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, four channels RGBA).
 * \see nppiWarpAffine_8u_C1R
 */
NppStatus nppiWarpAffine_8u_AC4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, three planes).
 * \see nppiWarpAffine_8u_C1R
 */
NppStatus nppiWarpAffine_8u_P3R(const Npp8u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst[3], 
                                       int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                       int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, four planes).
 * \see nppiWarpAffine_8u_C1R
 */
NppStatus nppiWarpAffine_8u_P4R(const Npp8u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst[4], 
                                       int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                       int interpolation);


/**
 * Inverse affine transform of an image (8bit unsigned integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpAffineBack. The function operates on source and destination regions of interest.
 * The affine warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * C_{00} * X_{new} + C_{01} * Y_{new} + C_{02} = x  \qquad
 * C_{10} * X_{new} + C_{11} * Y_{new} + C_{12} = y
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Affine transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpAffineBack_8u_C1R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                           int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                           int interpolation);


/**
 * Inverse affine transform of an image (8bit unsigned integer, three channels).
 * \see nppiWarpAffineBack_8u_C1R
 */
NppStatus nppiWarpAffineBack_8u_C3R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                           int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                           int interpolation);


/**
 * Inverse affine transform of an image (8bit unsigned integer, four channels).
 * \see nppiWarpAffineBack_8u_C1R
 */
NppStatus nppiWarpAffineBack_8u_C4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                           int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                           int interpolation);


/**
 * Inverse affine transform of an image (8bit unsigned integer, four channels RGBA).
 * \see nppiWarpAffineBack_8u_C1R
 */
NppStatus nppiWarpAffineBack_8u_AC4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (8bit unsigned integer, three planes).
 * \see nppiWarpAffineBack_8u_C1R
 */
NppStatus nppiWarpAffineBack_8u_P3R(const Npp8u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst[3], 
                                           int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                           int interpolation);


/**
 * Inverse affine transform of an image (8bit unsigned integer, four planes).
 * \see nppiWarpAffineBack_8u_C1R
 */
NppStatus nppiWarpAffineBack_8u_P4R(const Npp8u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst[4], 
                                           int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                           int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, single channel). This function performs affine warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the destination image. The function nppiWarpAffineQuad uses the same
 * formulas for pixel mapping as in nppiWarpAffine function. The transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param srcQuad               [IN] Source quadrangle
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param dstQuad               [IN] Destination quadrangle
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 * \return NPP_AFFINE_QUAD_INCORRECT_WARNING        Indicates a warning when quad doesn't conform to the transform properties. Fourth vertex is ignored, internally computed coordinates are used instead
 */
NppStatus nppiWarpAffineQuad_8u_C1R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst, int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, three channels).
 * \see nppiWarpAffineQuad_8u_C1R
 */
NppStatus nppiWarpAffineQuad_8u_C3R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst, int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, four channels).
 * \see nppiWarpAffineQuad_8u_C1R
 */
NppStatus nppiWarpAffineQuad_8u_C4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst, int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, four channels RGBA).
 * \see nppiWarpAffineQuad_8u_C1R
 */
NppStatus nppiWarpAffineQuad_8u_AC4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp8u* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, three planes).
 * \see nppiWarpAffineQuad_8u_C1R
 */
NppStatus nppiWarpAffineQuad_8u_P3R(const Npp8u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst[3], int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (8bit unsigned integer, four planes).
 * \see nppiWarpAffineQuad_8u_C1R
 */
NppStatus nppiWarpAffineQuad_8u_P4R(const Npp8u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst[4], int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly. The function operates on source and destination regions 
 * of interest. The affine warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Affine transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpAffine_16u_C1R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, three channels)
 * \see nppiWarpAffine_16u_C1R
 */
NppStatus nppiWarpAffine_16u_C3R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, four channels)
 * \see nppiWarpAffine_16u_C1R
 */
NppStatus nppiWarpAffine_16u_C4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, four channels RGBA)
 * \see nppiWarpAffine_16u_C1R
 */
NppStatus nppiWarpAffine_16u_AC4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                         int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                         int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, three planes)
 * \see nppiWarpAffine_16u_C1R
 */
NppStatus nppiWarpAffine_16u_P3R(const Npp16u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst[3], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, four planes)
 * \see nppiWarpAffine_16u_C1R
 */
NppStatus nppiWarpAffine_16u_P4R(const Npp16u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst[4], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Inverse affine transform of an image (16bit unsigned integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpAffineBack. The function operates on source and destination regions of interest.
 * The affine warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * C_{00} * X_{new} + C_{01} * Y_{new} + C_{02} = x  \qquad
 * C_{10} * X_{new} + C_{11} * Y_{new} + C_{12} = y
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Affine transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpAffineBack_16u_C1R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (16bit unsigned integer, three channels)
 * \see nppiWarpAffineBack_16u_C1R
 */
NppStatus nppiWarpAffineBack_16u_C3R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (16bit unsigned integer, four channels)
 * \see nppiWarpAffineBack_16u_C1R
 */
NppStatus nppiWarpAffineBack_16u_C4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (16bit unsigned integer, four channels RGBA)
 * \see nppiWarpAffineBack_16u_C1R
 */
NppStatus nppiWarpAffineBack_16u_AC4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                             int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                             int interpolation);


/**
 * Inverse affine transform of an image (16bit unsigned integer, three planes)
 * \see nppiWarpAffineBack_16u_C1R
 */
NppStatus nppiWarpAffineBack_16u_P3R(const Npp16u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst[3], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (16bit unsigned integer, four planes)
 * \see nppiWarpAffineBack_16u_C1R
 */
NppStatus nppiWarpAffineBack_16u_P4R(const Npp16u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst[4], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, single channel). This function performs affine warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the destination image. The function nppiWarpAffineQuad uses the same
 * formulas for pixel mapping as in nppiWarpAffine function. The transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param srcQuad               [IN] Source quadrangle
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param dstQuad               [IN] Destination quadrangle
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 * \return NPP_AFFINE_QUAD_INCORRECT_WARNING        Indicates a warning when quad doesn't conform to the transform properties. Fourth vertex is ignored, internally computed coordinates are used instead
 */
NppStatus nppiWarpAffineQuad_16u_C1R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, three channels). 
 * \see nppiWarpAffineQuad_16u_C1R
 */
NppStatus nppiWarpAffineQuad_16u_C3R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, four channels). 
 * \see nppiWarpAffineQuad_16u_C1R
 */
NppStatus nppiWarpAffineQuad_16u_C4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, four channels RGBA). 
 * \see nppiWarpAffineQuad_16u_C1R
 */
NppStatus nppiWarpAffineQuad_16u_AC4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                             const double srcQuad[4][2], Npp16u* pDst, int dstStep, NppiRect dstRoi, 
                                             const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, three planes). 
 * \see nppiWarpAffineQuad_16u_C1R
 */
NppStatus nppiWarpAffineQuad_16u_P3R(const Npp16u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst[3], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (16bit unsigned integer, four planes). 
 * \see nppiWarpAffineQuad_16u_C1R
 */
NppStatus nppiWarpAffineQuad_16u_P4R(const Npp16u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst[4], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit float, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly. The function operates on source and destination regions 
 * of interest. The affine warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with destination ROI specification.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Affine transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpAffine_32f_C1R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (32bit float, three channels).
 * \see nppiWarpAffine_32f_C1R
 */
NppStatus nppiWarpAffine_32f_C3R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (32bit float, four channels).
 * \see nppiWarpAffine_32f_C1R
 */
NppStatus nppiWarpAffine_32f_C4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (32bit float, four channels RGBA).
 * \see nppiWarpAffine_32f_C1R
 */
NppStatus nppiWarpAffine_32f_AC4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                         int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                         int interpolation);


/**
 * Affine transform of an image (32bit float, three planes).
 * \see nppiWarpAffine_32f_C1R
 */
NppStatus nppiWarpAffine_32f_P3R(const Npp32f* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst[3], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (32bit float, four planes).
 * \see nppiWarpAffine_32f_C1R
 */
NppStatus nppiWarpAffine_32f_P4R(const Npp32f* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst[4], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Inverse affine transform of an image (32bit float, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpAffineBack. The function operates on source and destination regions of interest.
 * The affine warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * C_{00} * X_{new} + C_{01} * Y_{new} + C_{02} = x  \qquad
 * C_{10} * X_{new} + C_{11} * Y_{new} + C_{12} = y
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with destination ROI specification.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Affine transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpAffineBack_32f_C1R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (32bit float, three channels).
 * \see nppiWarpAffineBack_32f_C1R
 */
NppStatus nppiWarpAffineBack_32f_C3R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (32bit float, four channels).
 * \see nppiWarpAffineBack_32f_C1R
 */
NppStatus nppiWarpAffineBack_32f_C4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (32bit float, four channels RGBA).
 * \see nppiWarpAffineBack_32f_C1R
 */
NppStatus nppiWarpAffineBack_32f_AC4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                             int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                             int interpolation);


/**
 * Inverse affine transform of an image (32bit float, three planes).
 * \see nppiWarpAffineBack_32f_C1R
 */
NppStatus nppiWarpAffineBack_32f_P3R(const Npp32f* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst[3], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (32bit float, four planes).
 * \see nppiWarpAffineBack_32f_C1R
 */
NppStatus nppiWarpAffineBack_32f_P4R(const Npp32f* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst[4], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Affine transform of an image (32bit float, single channel). This function performs affine warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the destination image. The function nppiWarpAffineQuad uses the same
 * formulas for pixel mapping as in nppiWarpAffine function. The transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * 
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param srcQuad               [IN] Source quadrangle
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param dstQuad               [IN] Destination quadrangle
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_AFFINE_QUAD_INCORRECT_WARNING        Indicates a warning when quad doesn't conform to the transform properties. Fourth vertex is ignored, internally computed coordinates are used instead
 */
NppStatus nppiWarpAffineQuad_32f_C1R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit float, three channels).
 * \see nppiWarpAffineQuad_32f_C1R
 */
NppStatus nppiWarpAffineQuad_32f_C3R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit float, four channels).
 * \see nppiWarpAffineQuad_32f_C1R
 */
NppStatus nppiWarpAffineQuad_32f_C4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit float, four channels RGBA).
 * \see nppiWarpAffineQuad_32f_C1R
 */
NppStatus nppiWarpAffineQuad_32f_AC4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                             const double srcQuad[4][2], Npp32f* pDst, int dstStep, NppiRect dstRoi, 
                                             const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit float, three planes).
 * \see nppiWarpAffineQuad_32f_C1R
 */
NppStatus nppiWarpAffineQuad_32f_P3R(const Npp32f* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst[3], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit float, four planes).
 * \see nppiWarpAffineQuad_32f_C1R
 */
NppStatus nppiWarpAffineQuad_32f_P4R(const Npp32f* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst[4], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit signed integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly. The function operates on source and destination regions 
 * of interest. The affine warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with destination ROI specification.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Affine transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpAffine_32s_C1R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (32bit signed integer, three channels).
 * \see nppiWarpAffine_32s_C1R
 */
NppStatus nppiWarpAffine_32s_C3R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (32bit signed integer, four channels).
 * \see nppiWarpAffine_32s_C1R
 */
NppStatus nppiWarpAffine_32s_C4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (32bit signed integer, four channels RGBA).
 * \see nppiWarpAffine_32s_C1R
 */
NppStatus nppiWarpAffine_32s_AC4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                         int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                         int interpolation);


/**
 * Affine transform of an image (32bit signed integer, three planes).
 * \see nppiWarpAffine_32s_C1R
 */
NppStatus nppiWarpAffine_32s_P3R(const Npp32s* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst[3], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Affine transform of an image (32bit signed integer, four planes).
 * \see nppiWarpAffine_32s_C1R
 */
NppStatus nppiWarpAffine_32s_P4R(const Npp32s* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst[4], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                        int interpolation);


/**
 * Inverse affine transform of an image (32bit signed integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpAffineBack. The function operates on source and destination regions of interest.
 * The affine warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * C_{00} * X_{new} + C_{01} * Y_{new} + C_{02} = x  \qquad
 * C_{10} * X_{new} + C_{11} * Y_{new} + C_{12} = y
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with destination ROI specification.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Affine transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpAffineBack_32s_C1R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (32bit signed integer, three channels).
 * \see nppiWarpAffineBack_32s_C1R
 */
NppStatus nppiWarpAffineBack_32s_C3R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (32bit signed integer, four channels).
 * \see nppiWarpAffineBack_32s_C1R
 */
NppStatus nppiWarpAffineBack_32s_C4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (32bit signed integer, four channels RGBA).
 * \see nppiWarpAffineBack_32s_C1R
 */
NppStatus nppiWarpAffineBack_32s_AC4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                             int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                             int interpolation);


/**
 * Inverse affine transform of an image (32bit signed integer, three planes).
 * \see nppiWarpAffineBack_32s_C1R
 */
NppStatus nppiWarpAffineBack_32s_P3R(const Npp32s* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst[3], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Inverse affine transform of an image (32bit signed integer, four planes).
 * \see nppiWarpAffineBack_32s_C1R
 */
NppStatus nppiWarpAffineBack_32s_P4R(const Npp32s* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst[4], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[2][3], 
                                            int interpolation);


/**
 * Affine transform of an image (32bit signed integer, single channel). This function performs affine warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the destination image. The function nppiWarpAffineQuad uses the same
 * formulas for pixel mapping as in nppiWarpAffine function. The transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * 
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param srcQuad               [IN] Source quadrangle
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param dstQuad               [IN] Destination quadrangle
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_AFFINE_QUAD_INCORRECT_WARNING        Indicates a warning when quad doesn't conform to the transform properties. Fourth vertex is ignored, internally computed coordinates are used instead
 */
NppStatus nppiWarpAffineQuad_32s_C1R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit signed integer, three channels).
 * \see nppiWarpAffineQuad_32s_C1R
 */
NppStatus nppiWarpAffineQuad_32s_C3R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit signed integer, four channels).
 * \see nppiWarpAffineQuad_32s_C1R
 */
NppStatus nppiWarpAffineQuad_32s_C4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit signed integer, four channels RGBA).
 * \see nppiWarpAffineQuad_32s_C1R
 */
NppStatus nppiWarpAffineQuad_32s_AC4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                             const double srcQuad[4][2], Npp32s* pDst, int dstStep, NppiRect dstRoi, 
                                             const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit signed integer, three planes).
 * \see nppiWarpAffineQuad_32s_C1R
 */
NppStatus nppiWarpAffineQuad_32s_P3R(const Npp32s* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst[3], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Affine transform of an image (32bit signed integer, four planes).
 * \see nppiWarpAffineQuad_32s_C1R
 */
NppStatus nppiWarpAffineQuad_32s_P4R(const Npp32s* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst[4], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);
//@}


/** @name Perspective warping, perspective transform calculation
 *  Perspective warping of an image is the transform of image pixel positions, defined by the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * C = \left[ \matrix{C_{00} & C_{01} & C_{02} \cr C_{10} & C_{11} & C_{12} \cr C_{20} & C_{21} & C_{22} } \right]
 * \f]
 * That is, any pixel of the transformed image with coordinates \f$(X_{new},Y_{new})\f$ has a preimage with 
 * coordinates \f$(x,y)\f$. The mapping \f$C\f$ is fully defined by 8 values \f$C_{ij}, (i,j)=\overline{0,2}\f$, except of \f$C_{22}\f$, which is a normalizer.
 * The transform has a property of mapping any convex quadrangle to a convex quadrangle, which is used in a group of functions nppiWarpPerspectiveQuad.
 * The NPPI implementation of perspective transform has some issues which are discussed in each function's documentation.
 */
//@{


/**
 * Calculates perspective transform coefficients given source rectangular ROI and its destination quadrangle projection
 *
 * \param srcRoi                 [IN] Source ROI
 * \param quad                   [IN] Destination quadrangle
 * \param coeffs                 [OUT] Perspective transform coefficients
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 */
NppStatus nppiGetPerspectiveTransform(NppiRect srcRoi, const double quad[4][2], double coeffs[3][3]);


/**
 * Calculates perspective transform projection of given source rectangular ROI
 *
 * \param srcRoi                 [IN] Source ROI
 * \param quad                   [OUT] Destination quadrangle
 * \param coeffs                 [IN] Perspective transform coefficients
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 */
NppStatus nppiGetPerspectiveQuad(NppiRect srcRoi, double quad[4][2], const double coeffs[3][3]);


/**
 * Calculates bounding box of the perspective transform projection of the given source rectangular ROI
 *
 * \param srcRoi                 [IN] Source ROI
 * \param bound                  [OUT] Bounding box of the transformed source ROI
 * \param coeffs                 [IN] Perspective transform coefficients
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 */
NppStatus nppiGetPerspectiveBound(NppiRect srcRoi, double bound[2][2], const double coeffs[3][3]);


/**
 * Perspective transform of an image (8bit unsigned integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set explicitly. The function operates on source and destination regions 
 * of interest. The perspective warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}}
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Perspective transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpPerspective_8u_C1R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                       int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                       int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, three channels).
 * \see nppiWarpPerspective_8u_C1R
 */
NppStatus nppiWarpPerspective_8u_C3R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                       int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                       int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, four channels).
 * \see nppiWarpPerspective_8u_C1R
 */
NppStatus nppiWarpPerspective_8u_C4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                       int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                       int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, four channels RGBA).
 * \see nppiWarpPerspective_8u_C1R
 */
NppStatus nppiWarpPerspective_8u_AC4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, three planes).
 * \see nppiWarpPerspective_8u_C1R
 */
NppStatus nppiWarpPerspective_8u_P3R(const Npp8u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst[3], 
                                       int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                       int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, four planes).
 * \see nppiWarpPerspective_8u_C1R
 */
NppStatus nppiWarpPerspective_8u_P4R(const Npp8u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst[4], 
                                       int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                       int interpolation);


/**
 * Inverse perspective transform of an image (8bit unsigned integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpPerspectiveBack. The function operates on source and destination regions of interest.
 * The perspective warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * \frac{C_{00} * X_{new} + C_{01} * Y_{new} + C_{02}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = x  \qquad
 * \frac{C_{10} * X_{new} + C_{11} * Y_{new} + C_{12}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = y
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Perspective transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpPerspectiveBack_8u_C1R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                           int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                           int interpolation);


/**
 * Inverse perspective transform of an image (8bit unsigned integer, three channels).
 * \see nppiWarpPerspectiveBack_8u_C1R
 */
NppStatus nppiWarpPerspectiveBack_8u_C3R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                           int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                           int interpolation);


/**
 * Inverse perspective transform of an image (8bit unsigned integer, four channels).
 * \see nppiWarpPerspectiveBack_8u_C1R
 */
NppStatus nppiWarpPerspectiveBack_8u_C4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                           int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                           int interpolation);


/**
 * Inverse perspective transform of an image (8bit unsigned integer, four channels RGBA).
 * \see nppiWarpPerspectiveBack_8u_C1R
 */
NppStatus nppiWarpPerspectiveBack_8u_AC4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (8bit unsigned integer, three planes).
 * \see nppiWarpPerspectiveBack_8u_C1R
 */
NppStatus nppiWarpPerspectiveBack_8u_P3R(const Npp8u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst[3], 
                                           int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                           int interpolation);


/**
 * Inverse perspective transform of an image (8bit unsigned integer, four planes).
 * \see nppiWarpPerspectiveBack_8u_C1R
 */
NppStatus nppiWarpPerspectiveBack_8u_P4R(const Npp8u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp8u* pDst[4], 
                                           int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                           int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, single channel). This function performs perspective warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the destination image. The function nppiWarpPerspectiveQuad uses the same
 * formulas for pixel mapping as in nppiWarpPerspective function. The transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param srcQuad               [IN] Source quadrangle
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param dstQuad               [IN] Destination quadrangle
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpPerspectiveQuad_8u_C1R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst, int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, three channels).
 * \see nppiWarpPerspectiveQuad_8u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_8u_C3R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst, int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, four channels).
 * \see nppiWarpPerspectiveQuad_8u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_8u_C4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst, int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, four channels RGBA).
 * \see nppiWarpPerspectiveQuad_8u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_8u_AC4R(const Npp8u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp8u* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, three planes).
 * \see nppiWarpPerspectiveQuad_8u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_8u_P3R(const Npp8u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst[3], int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (8bit unsigned integer, four planes).
 * \see nppiWarpPerspectiveQuad_8u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_8u_P4R(const Npp8u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                           const double srcQuad[4][2], Npp8u* pDst[4], int dstStep, NppiRect dstRoi, 
                                           const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set explicitly. The function operates on source and destination regions 
 * of interest. The perspective warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}}
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Perspective transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpPerspective_16u_C1R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, three channels)
 * \see nppiWarpPerspective_16u_C1R
 */
NppStatus nppiWarpPerspective_16u_C3R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, four channels)
 * \see nppiWarpPerspective_16u_C1R
 */
NppStatus nppiWarpPerspective_16u_C4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, four channels RGBA)
 * \see nppiWarpPerspective_16u_C1R
 */
NppStatus nppiWarpPerspective_16u_AC4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                         int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                         int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, three planes)
 * \see nppiWarpPerspective_16u_C1R
 */
NppStatus nppiWarpPerspective_16u_P3R(const Npp16u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst[3], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, four planes)
 * \see nppiWarpPerspective_16u_C1R
 */
NppStatus nppiWarpPerspective_16u_P4R(const Npp16u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst[4], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Inverse perspective transform of an image (16bit unsigned integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpPerspectiveBack. The function operates on source and destination regions of interest.
 * The perspective warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * \frac{C_{00} * X_{new} + C_{01} * Y_{new} + C_{02}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = x  \qquad
 * \frac{C_{10} * X_{new} + C_{11} * Y_{new} + C_{12}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = y
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Perspective transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpPerspectiveBack_16u_C1R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (16bit unsigned integer, three channels)
 * \see nppiWarpPerspectiveBack_16u_C1R
 */
NppStatus nppiWarpPerspectiveBack_16u_C3R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (16bit unsigned integer, four channels)
 * \see nppiWarpPerspectiveBack_16u_C1R
 */
NppStatus nppiWarpPerspectiveBack_16u_C4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (16bit unsigned integer, four channels RGBA)
 * \see nppiWarpPerspectiveBack_16u_C1R
 */
NppStatus nppiWarpPerspectiveBack_16u_AC4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst, 
                                             int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                             int interpolation);


/**
 * Inverse perspective transform of an image (16bit unsigned integer, three planes)
 * \see nppiWarpPerspectiveBack_16u_C1R
 */
NppStatus nppiWarpPerspectiveBack_16u_P3R(const Npp16u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst[3], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (16bit unsigned integer, four planes)
 * \see nppiWarpPerspectiveBack_16u_C1R
 */
NppStatus nppiWarpPerspectiveBack_16u_P4R(const Npp16u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp16u* pDst[4], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, single channel). This function performs perspective warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the destination image. The function nppiWarpPerspectiveQuad uses the same
 * formulas for pixel mapping as in nppiWarpPerspective function. The transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side. However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param srcQuad               [IN] Source quadrangle
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param dstQuad               [IN] Destination quadrangle
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 * \return NPP_MISALIGNED_DST_ROI_WARNING           Indicates a warning that the speed of primitive execution was reduced due to destination ROI misalignment
 */
NppStatus nppiWarpPerspectiveQuad_16u_C1R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, three channels). 
 * \see nppiWarpPerspectiveQuad_16u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_16u_C3R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, four channels). 
 * \see nppiWarpPerspectiveQuad_16u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_16u_C4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, four channels RGBA). 
 * \see nppiWarpPerspectiveQuad_16u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_16u_AC4R(const Npp16u* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                             const double srcQuad[4][2], Npp16u* pDst, int dstStep, NppiRect dstRoi, 
                                             const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, three planes). 
 * \see nppiWarpPerspectiveQuad_16u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_16u_P3R(const Npp16u* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst[3], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (16bit unsigned integer, four planes). 
 * \see nppiWarpPerspectiveQuad_16u_C1R
 */
NppStatus nppiWarpPerspectiveQuad_16u_P4R(const Npp16u* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp16u* pDst[4], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit float, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set explicitly. The function operates on source and destination regions 
 * of interest. The perspective warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}}
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help with destination ROI specification.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Perspective transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpPerspective_32f_C1R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (32bit float, three channels).
 * \see nppiWarpPerspective_32f_C1R
 */
NppStatus nppiWarpPerspective_32f_C3R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (32bit float, four channels).
 * \see nppiWarpPerspective_32f_C1R
 */
NppStatus nppiWarpPerspective_32f_C4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (32bit float, four channels RGBA).
 * \see nppiWarpPerspective_32f_C1R
 */
NppStatus nppiWarpPerspective_32f_AC4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                         int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                         int interpolation);


/**
 * Perspective transform of an image (32bit float, three planes).
 * \see nppiWarpPerspective_32f_C1R
 */
NppStatus nppiWarpPerspective_32f_P3R(const Npp32f* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst[3], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (32bit float, four planes).
 * \see nppiWarpPerspective_32f_C1R
 */
NppStatus nppiWarpPerspective_32f_P4R(const Npp32f* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst[4], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Inverse perspective transform of an image (32bit float, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpPerspectiveBack. The function operates on source and destination regions of interest.
 * The perspective warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * \frac{C_{00} * X_{new} + C_{01} * Y_{new} + C_{02}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = x  \qquad
 * \frac{C_{10} * X_{new} + C_{11} * Y_{new} + C_{12}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = y
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help with destination ROI specification.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Perspective transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpPerspectiveBack_32f_C1R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (32bit float, three channels).
 * \see nppiWarpPerspectiveBack_32f_C1R
 */
NppStatus nppiWarpPerspectiveBack_32f_C3R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (32bit float, four channels).
 * \see nppiWarpPerspectiveBack_32f_C1R
 */
NppStatus nppiWarpPerspectiveBack_32f_C4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (32bit float, four channels RGBA).
 * \see nppiWarpPerspectiveBack_32f_C1R
 */
NppStatus nppiWarpPerspectiveBack_32f_AC4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst, 
                                             int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                             int interpolation);


/**
 * Inverse perspective transform of an image (32bit float, three planes).
 * \see nppiWarpPerspectiveBack_32f_C1R
 */
NppStatus nppiWarpPerspectiveBack_32f_P3R(const Npp32f* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst[3], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (32bit float, four planes).
 * \see nppiWarpPerspectiveBack_32f_C1R
 */
NppStatus nppiWarpPerspectiveBack_32f_P4R(const Npp32f* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32f* pDst[4], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Perspective transform of an image (32bit float, single channel). This function performs perspective warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the destination image. The function nppiWarpPerspectiveQuad uses the same
 * formulas for pixel mapping as in nppiWarpPerspective function. The transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * 
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param srcQuad               [IN] Source quadrangle
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param dstQuad               [IN] Destination quadrangle
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpPerspectiveQuad_32f_C1R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit float, three channels).
 * \see nppiWarpPerspectiveQuad_32f_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32f_C3R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit float, four channels).
 * \see nppiWarpPerspectiveQuad_32f_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32f_C4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit float, four channels RGBA).
 * \see nppiWarpPerspectiveQuad_32f_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32f_AC4R(const Npp32f* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                             const double srcQuad[4][2], Npp32f* pDst, int dstStep, NppiRect dstRoi, 
                                             const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit float, three planes).
 * \see nppiWarpPerspectiveQuad_32f_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32f_P3R(const Npp32f* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst[3], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit float, four planes).
 * \see nppiWarpPerspectiveQuad_32f_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32f_P4R(const Npp32f* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32f* pDst[4], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set explicitly. The function operates on source and destination regions 
 * of interest. The perspective warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}}
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help with destination ROI specification.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Perspective transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpPerspective_32s_C1R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, three channels).
 * \see nppiWarpPerspective_32s_C1R
 */
NppStatus nppiWarpPerspective_32s_C3R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, four channels).
 * \see nppiWarpPerspective_32s_C1R
 */
NppStatus nppiWarpPerspective_32s_C4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, four channels RGBA).
 * \see nppiWarpPerspective_32s_C1R
 */
NppStatus nppiWarpPerspective_32s_AC4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                         int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                         int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, three planes).
 * \see nppiWarpPerspective_32s_C1R
 */
NppStatus nppiWarpPerspective_32s_P3R(const Npp32s* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst[3], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, four planes).
 * \see nppiWarpPerspective_32s_C1R
 */
NppStatus nppiWarpPerspective_32s_P4R(const Npp32s* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst[4], 
                                        int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                        int interpolation);


/**
 * Inverse perspective transform of an image (32bit signed integer, single channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpPerspectiveBack. The function operates on source and destination regions of interest.
 * The perspective warp function transforms the source image pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * \frac{C_{00} * X_{new} + C_{01} * Y_{new} + C_{02}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = x  \qquad
 * \frac{C_{10} * X_{new} + C_{11} * Y_{new} + C_{12}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = y
 * \f]
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help with destination ROI specification.
 *
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param coeffs                [IN] Perspective transform coefficients
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpPerspectiveBack_32s_C1R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (32bit signed integer, three channels).
 * \see nppiWarpPerspectiveBack_32s_C1R
 */
NppStatus nppiWarpPerspectiveBack_32s_C3R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (32bit signed integer, four channels).
 * \see nppiWarpPerspectiveBack_32s_C1R
 */
NppStatus nppiWarpPerspectiveBack_32s_C4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (32bit signed integer, four channels RGBA).
 * \see nppiWarpPerspectiveBack_32s_C1R
 */
NppStatus nppiWarpPerspectiveBack_32s_AC4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst, 
                                             int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                             int interpolation);


/**
 * Inverse perspective transform of an image (32bit signed integer, three planes).
 * \see nppiWarpPerspectiveBack_32s_C1R
 */
NppStatus nppiWarpPerspectiveBack_32s_P3R(const Npp32s* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst[3], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Inverse perspective transform of an image (32bit signed integer, four planes).
 * \see nppiWarpPerspectiveBack_32s_C1R
 */
NppStatus nppiWarpPerspectiveBack_32s_P4R(const Npp32s* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, Npp32s* pDst[4], 
                                            int dstStep, NppiRect dstRoi, const double coeffs[3][3], 
                                            int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, single channel). This function performs perspective warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the destination image. The function nppiWarpPerspectiveQuad uses the same
 * formulas for pixel mapping as in nppiWarpPerspective function. The transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified interpolation method and written to the destination ROI.
 * 
 * \param pSrc                  [IN] Pointer to source image data
 * \param srcSize               [IN] Size of source image in pixels
 * \param srcStep               [IN] Step in bytes between any pair of sequential rows of source image
 * \param srcRoi                [IN] Source ROI
 * \param srcQuad               [IN] Source quadrangle
 * \param pDst                  [OUT] Pointer to destination image data 
 * \param dstStep               [IN] Step in bytes between any pair of sequential rows of destination image
 * \param dstRoi                [IN] Destination ROI
 * \param dstQuad               [IN] Destination quadrangle
 * \param interpolation         [IN] Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 *
 * \return NPP_SUCCESS                              Indicates no error. Any other value indicates an error or a warning
 * \return NPP_NULL_POINTER_ERROR                   Indicates an error condition if one of the specified pointers is NULL
 * \return NPP_SIZE_ERROR                           Indicates an error condition if any image dimension has zero or negative value
 * \return NPP_STEP_ERROR                           Indicates an error condition if srcStep or dstStep has a zero or negative value
 * \return NPP_RECT_ERROR                           Indicates an error condition if width or height of the intersection of the srcRoi and source image is less than or equal to 1
 * \return NPP_WRONG_INTERSECTION_ROI_ERROR         Indicates an error condition if srcRoi has no intersection with the source image
 * \return NPP_INTERPOLATION_ERROR                  Indicates an error condition if interpolation has an illegal value
 * \return NPP_COEFF_ERROR                          Indicates an error condition if coefficient values are invalid
 * \return NPP_ALIGNMENT_ERROR                      Indicates an error condition if any of input surfaces is not 4-byte aligned
 * \return NPP_CUDA_KERNEL_EXECUTION_ERROR          Indicates an error condition if cudaGetLastError() returns an error right after kernel execution
 * \return NPP_WRONG_INTERSECTION_QUAD_WARNING      Indicates a warning that no operation is performed if the transformed source ROI has no intersection with the destination ROI
 */
NppStatus nppiWarpPerspectiveQuad_32s_C1R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, three channels).
 * \see nppiWarpPerspectiveQuad_32s_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32s_C3R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, four channels).
 * \see nppiWarpPerspectiveQuad_32s_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32s_C4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst, int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, four channels RGBA).
 * \see nppiWarpPerspectiveQuad_32s_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32s_AC4R(const Npp32s* pSrc, NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                             const double srcQuad[4][2], Npp32s* pDst, int dstStep, NppiRect dstRoi, 
                                             const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, three planes).
 * \see nppiWarpPerspectiveQuad_32s_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32s_P3R(const Npp32s* pSrc[3], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst[3], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);


/**
 * Perspective transform of an image (32bit signed integer, four planes).
 * \see nppiWarpPerspectiveQuad_32s_C1R
 */
NppStatus nppiWarpPerspectiveQuad_32s_P4R(const Npp32s* pSrc[4], NppiSize srcSize, int srcStep, NppiRect srcRoi, 
                                            const double srcQuad[4][2], Npp32s* pDst[4], int dstStep, NppiRect dstRoi, 
                                            const double dstQuad[4][2], int interpolation);
//@}

/*@}*/ // end of nppi module
 
#ifdef __cplusplus
} // extern "C"
#endif

#endif // NV_NPPI_H
